/** 
 * @file classConfig.h
 *
 * Defini la configuration de chaque module du robot pour les logs:
 * permet de savoir quel est le niveau de debug de chaque partie de code
 */

#ifndef __CLASS_CONFIG_H__
#define __CLASS_CONFIG_H__

#include "robotTypes.h"

// ===========================================================================
// enum ClassId
// ===========================================================================

/** @brief Liste de tous les noms de classe */
typedef enum ClassId {
  CLASS_DEFAULT=0,

  CLASS_CONFIG,
  CLASS_ROBOT_BASE,
  CLASS_ROBOT_COMPONENT,
  CLASS_ROBOT_TIMER,

  CLASS_LOCK,
  CLASS_SOCKET,
  CLASS_MTHREAD,
  CLASS_SHM,
  CLASS_VIDEO,
  CLASS_BACKUP,
  CLASS_AMBIANCE,
  CLASS_IPAQ,
  CLASS_SERIAL_PORT,
  CLASS_EVENTS_MANAGER,
  
  CLASS_UART_MANAGER,
  CLASS_UART,
  CLASS_DRIDAQ,
  CLASS_IO_MANAGER,
  CLASS_BUTTON,
  CLASS_LCD,
  CLASS_SERVO,
  CLASS_COLOR,
  CLASS_ENV_DETECTOR_03,
  CLASS_ODOMETER,
  CLASS_GONIO,
  CLASS_ENV_DETECTOR, 
  CLASS_CATAPULT,
  CLASS_K2000,
  CLASS_VOLTAGE,
  CLASS_EYES,

  CLASS_MOVEMENT_MANAGER,
  CLASS_MOVE,
  CLASS_ROBOT_POSITION,
  CLASS_MOVEMENT,
  CLASS_MOTOR,
  
  CLASS_SIMULATOR,
  CLASS_OBJ_SIMU,
  CLASS_VIEWER_3D,
  CLASS_LOG,
  CLASS_SOUND,

  CLASS_ROBOT_MAIN,
  CLASS_STRATEGY,
  CLASS_HLI,
  CLASS_TRAJGEN,
  CLASS_SHORTEST_WAY,
  CLASS_EXPLORE,

  CLASS_NBR
} ClassId;

// ===========================================================================
// class ClassConfig
// ===========================================================================

/**
 * @class ClassConfig 
 * Parametres qui configurent une class
 */
class ClassConfig {
 private:
  VerboseLevel verbose_;
  ClassId      id_;

 public:
  ClassConfig(ClassId id);
  /** @brief Retourne l'id de la classe */
  ClassId             id() const;
  /** @brief Retourne le niveau de debug */
  VerboseLevel        verbose() const;
  /** @brief met a jour le niveau de debug */
  void                setVerboseLevel(VerboseLevel level);
  /** @brief Renvoie la classConfig correspondant a un id donne */
  static ClassConfig* find(ClassId id);
};

// ===========================================================================
// inline functions
// ===========================================================================

// ---------------------------------------------------------------------------
// ClassConfig::ClassConfig
// ---------------------------------------------------------------------------
inline ClassConfig::ClassConfig(ClassId id) :
    verbose_(VERBOSE_DEFAULT), id_(id) 
{
}

// ---------------------------------------------------------------------------
// ClassConfig::verbose
// ---------------------------------------------------------------------------
inline VerboseLevel ClassConfig::verbose() const
{
    return verbose_;
}

// ---------------------------------------------------------------------------
// ClassConfig::setVerboseLevel
// ---------------------------------------------------------------------------
inline void ClassConfig::setVerboseLevel(VerboseLevel level)
{
    verbose_ = level;
}

// ---------------------------------------------------------------------------
// ClassConfig::setVerboseLevel
// ---------------------------------------------------------------------------
inline ClassId ClassConfig::id()
{
    return id_;
}



#endif // __CLASS_CONFIG_H__
