#include <stdio.h>
#include <GL/glut.h>

#include "robotMacro.h"
#include "logPlayer.h"
#include "gltools.h"
#include "viewer3D.h"
#include "robotConst.h"
#include "robotMacro.h"

#include "text.h"

// ==========================================================================
// Player control
// ==========================================================================

static ViewerText* text0_=NULL;
static ViewerText* text1_=NULL;
static ViewerText* text2_=NULL;
static int winId_=0;
static Viewer3D* viewer_=NULL;
static Millisecond time_=0;
static Millisecond matchTime_=0;
static char info_[255];
static bool init_=false;
static double scaleX=1;
static double scaleY=1;

struct ControlButton {
  TextureId texture1;
  TextureId texturePushed;
  const char* name;
  void (*clickCB)();
  int x;
  int y;
  bool enable;
  bool clicked;

  void click(int mx, int my) {
    if (mx<x || mx>x+32 || my<y || my>y+32) return;
    clicked=true;
  }

  void unclick(int mx, int my) {
    if (mx<x || mx>x+32 || my<y || my>y+32) return;
    if (enable && clickCB) clickCB();
    clicked=false;
  }
};
static ControlButton btn_[CTRL_BTN_NBR]= {
  { TEX_BTN_PREV_0, TEX_BTN_PREV_1, "", NULL, 
    0, 0, true, false},
  { TEX_BTN_STEP_BACKWARD_0, TEX_BTN_STEP_BACKWARD_1, "", NULL, 
    32, 0, true, false},
  { TEX_BTN_PLAY_0, TEX_BTN_PLAY_1, "", NULL, 
    64, 0, true, false},
  { TEX_BTN_STEP_FORWARD_0, TEX_BTN_STEP_FORWARD_1, "", NULL, 
    96, 0, true, false},
  { TEX_BTN_NEXT_0, TEX_BTN_NEXT_1, "", NULL, 
    128, 0, true, false},
  { TEX_BTN_SLOWER_0, TEX_BTN_SLOWER_1, "", NULL, 
    170, 0, true, false},
  { TEX_BTN_FASTER_0, TEX_BTN_FASTER_1, "", NULL, 
    202, 0, true, false},
  { TEX_BTN_REC_START_0, TEX_BTN_REC_START_1, "", NULL, 
    244, 0, true, false}
};

void playerControlRecordBtnCB(ControlButtonId btn, 
			      void (*clickCB)())
{
  btn_[btn].clickCB = clickCB;
}

void playerControlSetBtnEnable(ControlButtonId btn, 
			       bool enable)
{
  btn_[btn].enable=enable;
}

void playerControlSetBtnTexture(ControlButtonId btn, 
				TextureId t1, 
				TextureId t2)
{
  btn_[btn].texture1=t1;
  btn_[btn].texturePushed=t2;
}

void playerControlSetTimes(Millisecond realTime, 
			   Millisecond matchTime,
			   const char* info)
{
  time_      = realTime;
  matchTime_ = matchTime;
  strcpy(info_, info);
}

// --------------------------------------------------------------------
// keyboardPlayerControl
// --------------------------------------------------------------------
void keyboardPlayerControl(unsigned char key, int x, int y)
{ 
  switch (key) {
  default:
    break;
  }
}

// --------------------------------------------------------------------
// menuPlayerControl
// --------------------------------------------------------------------
void menuPlayerControl(int value)
{
  switch(value) {
  default:
    break;
  }
}

// --------------------------------------------------------------------
// reshapePlayerControl
// --------------------------------------------------------------------
void reshapePlayerControl(int w, int h)
{
  glViewport(0,0,
	     (GLsizei) (w), 
	     (GLsizei) (h)); 
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  scaleX=276./w;
  scaleY=100./h;
  glOrtho(0.0f, scaleX*w, scaleY*h, 0.0f,-1.0f,1.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

// ---------------------------------------------------------------------------
// displayPlayerControl
// ---------------------------------------------------------------------------
void displayPlayerControl() 
{
  if (winId_==0) return;
  glutSetWindow(winId_);

  glLoadIdentity();
  glClear (GL_COLOR_BUFFER_BIT);// Reset The Modelview Matrix
  glClear (GL_DEPTH_BUFFER_BIT);// Reset The Modelview Matrix

  // draw buttons
  for(int i=0; i<CTRL_BTN_NBR; i++) {
    if (btn_[i].enable) {
      glColor3f(1.0f,1.0f,1.0f);
    } else {
      glColor3f(.5f,.5f,.5f);
    }
    if (viewer_ && viewer_->texture_) {
      if (btn_[i].clicked) {
	viewer_->texture_->apply(btn_[i].texturePushed);
      } else {
	viewer_->texture_->apply(btn_[i].texture1);
      }
    }
    glBegin(GL_QUADS);
    glTexCoord2f( 0.0, 0.0 ); glVertex2d(btn_[i].x+31, btn_[i].y);
    glTexCoord2f( 0.0, 1.0 ); glVertex2d(btn_[i].x+31, btn_[i].y+31);
    glTexCoord2f( 1.0, 1.0 ); glVertex2d(btn_[i].x,    btn_[i].y+31);	
    glTexCoord2f( 1.0, 0.0 ); glVertex2d(btn_[i].x,    btn_[i].y);	
    glEnd(); 
    glDisable(GL_TEXTURE_2D);
  }

  // draw timers
  text0_->draw(5, 50, "Log Time: %d'%02d\"%03d", 
	      time_/60000, (time_/1000)%60, time_%1000);
  text1_->draw(5, 65, "Match Time: %d'%02d\"%03d", 
	      matchTime_/60000, (matchTime_/1000)%60, matchTime_%1000);
  text2_->draw(5, 80, "%s", info_);
  glutSwapBuffers();
}

// --------------------------------------------------------------------
// mousePlayerControl
// --------------------------------------------------------------------
void mousePlayerControl(int button, int state, int x, int y)
{
  int i=0;
  if (button == GLUT_LEFT_BUTTON) {
    if (state == GLUT_DOWN) {
      for(i=0;i<CTRL_BTN_NBR;i++) {
	btn_[i].click((int)(x*scaleX), (int)(y*scaleY));
      }
    } else {
      for(i=0;i<CTRL_BTN_NBR;i++) {
	btn_[i].unclick((int)(x*scaleX), (int)(y*scaleY));
      }
    }
  }
}

// --------------------------------------------------------------------
// initPlayerControl
// --------------------------------------------------------------------
void initPlayerControl(int winId, Viewer3D* viewer) 
{
  if (!init_) {
    winId_=winId; 
    viewer_=viewer;
    text0_=new ViewerText(10, ViewerColor(0,0.9,0,1));
    text1_=new ViewerText(10, ViewerColor(1,1,0,1));
    text2_=new ViewerText(10, ViewerColor(1,1,1,1));
    glEnable(GL_BLEND);
    glShadeModel (GL_SMOOTH);
    glClearColor (0.0, 0.0, 0.0, 0.0);
    glClear (GL_COLOR_BUFFER_BIT);
    init_=true;
  }
}
