#include <stdio.h>
#include <GL/glut.h>
#include <math.h>
#include "mthread.h"

#ifndef M_PI
#define M_PI (3.14)
#define M_PI_2 (M_PI/2.)
#define M_3_PI_2 (M_PI_2*3.)
#endif

#include "robot3d.h"
#include "gltools.h"
#include "viewer3D.h"
#include "texture.h"


extern pthread_mutex_t viewer3DrepositoryLock;

// -------------------------------------------------------------------------
// Private namespace
// -------------------------------------------------------------------------
namespace {
    GLdouble xEye=0*SCALE3D, yEye=-1000*SCALE3D, zEye=1000*SCALE3D;
    GLdouble xCenter=1500*SCALE3D, yCenter=1050*SCALE3D, zCenter=0*SCALE3D; 
    GLdouble xUp=0.1, yUp=0.1, zUp=1, dirR=M_PI_2;
    bool screenshot=false;
    int W, H;
#define ZNEAR 1
#define ZFAR 10000
#define FOV_START 60
    int FOVY = FOV_START;
    GLdouble dAspect=1;

    int winId_=0;
    Viewer3D* viewer_=NULL;
    bool init_=false;
    bool cameraModeAuto_ = false;

    // -----------------------------------------------------------------------
    // SetTargetPosition -- Eyes direction
    // -----------------------------------------------------------------------
    void setTargetPosition(Millimeter cx, Millimeter cy, Millimeter cz)
    {
	xCenter = cx*SCALE3D;
	yCenter = cy*SCALE3D;
	zCenter = cz*SCALE3D;
    
	if(xEye != xCenter){
	    dirR = atan((yEye-yCenter)/(xEye-xCenter));
	    if(xCenter<xEye) dirR += M_PI;
	}
	else{
	    if(yCenter>yEye) dirR = M_PI_2;
	    else dirR = M_PI_2*3;
	}
    } // setTargetPosition
  
    // -------------------------------------------------------------------------
    // setEyesPosition
    // -------------------------------------------------------------------------
    void setEyesPosition(Millimeter cx, Millimeter cy, Millimeter cz)
    {
	xEye = cx*SCALE3D;
	yEye = cy*SCALE3D;
	zEye = cz*SCALE3D;
    } // setTargetPosition
  
    // -------------------------------------------------------------------------
    // eyesPositionRotate
    // -------------------------------------------------------------------------
    void eyesPositionRotate(int mode)
    {
	double d=sqrt((xEye-xCenter)*(xEye-xCenter)+(yEye-yCenter)*(yEye-yCenter));
	switch(mode) {
	case 0:
	    dirR+=M_PI/16.;
	    break;
	case 1:
	    dirR-=M_PI/16.;
	    break;
	case 2:
	    d*=1.2;
	    break;
	case 3:
	    d*=0.8;
	    break;
	}
	xEye = xCenter+d*cos(dirR+M_PI);
	yEye = yCenter+d*sin(dirR+M_PI);
    } // setTargetPosition

    // -------------------------------------------------------------------------
    // eyesNearestCorner
    // -------------------------------------------------------------------------
    void eyesNearestCorner() 
    {
	Point c1(-300,-300);
	Point c2(3300,-300);
	Point c3(3300,2400);
	Point c4(-300,2400);
	Point c5(-300,1050);
	Point c6(1500,-300);
	Point c7(1500,2400);
	Point c8(3300,1050);
	Point t(xCenter/SCALE3D, yCenter/SCALE3D);

	Millimeter dist1=t.dist(c1);
	xEye=c2.x*SCALE3D;
	yEye=c2.y*SCALE3D;
	zEye=700*SCALE3D;
	Millimeter bestDist=dist1;
    
	dist1=t.dist(c2);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c2.x*SCALE3D;
	    yEye=c2.y*SCALE3D;
	}
    
	dist1=t.dist(c3);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c3.x*SCALE3D;
	    yEye=c3.y*SCALE3D;
	}
    
	dist1=t.dist(c4);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c4.x*SCALE3D;
	    yEye=c4.y*SCALE3D;
	}
    
	dist1=t.dist(c5);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c5.x*SCALE3D;
	    yEye=c5.y*SCALE3D;
	}
    
	dist1=t.dist(c6);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c6.x*SCALE3D;
	    yEye=c6.y*SCALE3D;
	}

	dist1=t.dist(c7);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c7.x*SCALE3D;
	    yEye=c7.y*SCALE3D;
	}
    
	dist1=t.dist(c8);
	if (dist1<bestDist) {
	    bestDist=dist1;
	    xEye=c8.x*SCALE3D;
	    yEye=c8.y*SCALE3D;
	}
	viewer_->setCameraPosition(xEye/SCALE3D, yEye/SCALE3D, zEye/SCALE3D);
    }
  
}

// -------------------------------------------------------------------------
// resetRobot3D
// -------------------------------------------------------------------------
void initRobot3D(int winId, Viewer3D* viewer) {
    if (!init_) {
	winId_=winId;
	
	viewer_=viewer;
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
    
	glShadeModel (GL_SMOOTH);
    
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glClear (GL_COLOR_BUFFER_BIT);
	glClear (GL_DEPTH_BUFFER_BIT);
    
	glutCreateMenu (menuRobot3D);
	glutAddMenuEntry ("Vue du coin le plus proche", 1);
	glutAddMenuEntry ("Camera automatique/fixe",    2);
	glutAddMenuEntry ("Textures / pas de texture",  3);
	glutAttachMenu (GLUT_RIGHT_BUTTON);
	init_=true;
    }
    resetRobot3D();
}

// -------------------------------------------------------------------------
// resetRobot3D
// -------------------------------------------------------------------------
void resetRobot3D() 
{
  
}

// -------------------------------------------------------------------------
// displayRobot3D
// -------------------------------------------------------------------------
void displayRobot3D(void) 
{
    //Lock localLock(&viewer3DrepositoryLock);
    if (winId_==0) return;
    
    glutSetWindow(winId_);
    if (cameraModeAuto_) {
	eyesNearestCorner(); 
    }
    if (viewer_->robotRP_.center.x<10) {
	setTargetPosition(1500, 
			  1050, 0); // centre du terrain
    } else {
	setTargetPosition(viewer_->robotRP_.center.x, 
			  viewer_->robotRP_.center.y, 0);
    }
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(FOVY, dAspect, ZNEAR, ZFAR);
    gluLookAt (xEye, yEye, zEye, 
	       xCenter, yCenter, zCenter, 
	       cos(dirR), sin(dirR), 0);
    //gluLookAt (xEye, yEye, zEye, xCenter, yCenter, zCenter, xUp, yUp, zUp);
    glMatrixMode(GL_MODELVIEW);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity ();
    
    viewer_->drawBG();
    viewer_->drawPalmTrees();
    viewer_->drawBalls();
    // viewer_->drawObstacles();
    viewer_->drawEnnemy();
    viewer_->drawEnnemyRS();
    viewer_->drawRobotRS();
    viewer_->drawRobot();
    viewer_->drawFillets();
    glutSwapBuffers();
    Viewer3D::movieScreenShot(VIEWER_SCREEN_3D);
    // fprintf(stdout, "display3D\n");
}

// -------------------------------------------------------------------------
// keyboardRobot3D
// -------------------------------------------------------------------------
void keyboardRobot3D (unsigned char key, int x, int y)
{ 
    switch (key) {
    case 'd':  // enter
	eyesPositionRotate(0);
	glutPostRedisplay();
	break;
    case 'q':  // enter
	eyesPositionRotate(1);
	glutPostRedisplay();
	break;
    case 'z':  // enter
	eyesPositionRotate(2);
	glutPostRedisplay();
	break;
    case 's':  // enter
	eyesPositionRotate(3);
	glutPostRedisplay();
	break;
    case 'c':  // enter
	Viewer3D::screenShot("robot3d");
	break;
    default:
	break;
    }
}

// -------------------------------------------------------------------------
// menuRobot3D
// -------------------------------------------------------------------------
void menuRobot3D (int value)
{
    if (value==1) {
	eyesNearestCorner();
    }
    if (value==2) {
	cameraModeAuto_ = !cameraModeAuto_;
    }
    if (value==3) {
	viewer_->useTexture_ = ! viewer_->useTexture_;
    }
    
}


// -------------------------------------------------------------------------
// reshapeRobot3D
// -------------------------------------------------------------------------
void reshapeRobot3D(int w, int h)
{
    if (winId_==0) return;
    glutSetWindow(winId_);
    
    //Lock localLock(&viewer3DrepositoryLock);
    glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
  
    // quivalent du choix d'une lentille pour la camera	
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective(30.0, (GLfloat) w/(GLfloat) h, 1.0, 800.0);
  
    glMatrixMode(GL_MODELVIEW);
}

// -------------------------------------------------------------------------
// idleRobot3D
// -------------------------------------------------------------------------
void idleRobot3D(void)
{

}

// -------------------------------------------------------------------------
// timerRobot3D
// -------------------------------------------------------------------------
void timerRobot3D(int v)
{

}

// -------------------------------------------------------------------------
// setEyesPosition
// -------------------------------------------------------------------------
void setCameraPosition3D(Millimeter cx, Millimeter cy, Millimeter cz)
{
    setEyesPosition( cx, cy, cz);
}
