#include <stdio.h>
#ifdef linux
#include <GL/glut.h>
#endif

#include "robotData.h"
#include "gltools.h"
#include "viewer3D.h"
#include "robotConst.h"
#include "mthread.h"
#include "text.h"

extern pthread_mutex_t viewer3DrepositoryLock;

// ---------------------------------------------------------------------------
// Private namespace
// ---------------------------------------------------------------------------
namespace {

  Viewer3D* viewer_=NULL;
  int winId_=0;
  bool init_=false;
  unsigned int lastMenuLength=0;
  ViewerText* text_=NULL;

  Millisecond RobotDataTmin=0;
  Millisecond RobotDataTmax=90000;
  int RobotDataWindowHeight=140;
  int RobotDataWindowWidth=500;
  float RobotDataWindowMinH=10;
  float RobotDataWindowMinW=20;

  // -------------------------------------------------------------------------
  // drawGrid
  // -------------------------------------------------------------------------
  void drawGrid() {
    glLoadIdentity();
    glClear (GL_COLOR_BUFFER_BIT);// Reset The Modelview Matrix
    glClear (GL_DEPTH_BUFFER_BIT);// Reset The Modelview Matrix
    int i;
    // dessin du contour
    glColor3f(1.0f,1.0f,1.0f);
    glBegin(GL_LINE_STRIP);
    glVertex2d(RobotDataWindowMinW+RobotDataWindowWidth, 
	       RobotDataWindowMinH+RobotDataWindowHeight);	
    glVertex2d(RobotDataWindowMinW, 
	       RobotDataWindowMinH+RobotDataWindowHeight);
    glVertex2d(RobotDataWindowMinW, 
	       RobotDataWindowMinH);

    glEnd();		
    //dessin de la grille
    for(i=0;i<10;++i) {
	glBegin(GL_LINES);
	glVertex2d(RobotDataWindowMinW+i*RobotDataWindowWidth/10, 
		   RobotDataWindowMinH-2+RobotDataWindowHeight);
	glVertex2d(RobotDataWindowMinW+i*RobotDataWindowWidth/10, 
		   RobotDataWindowMinH+2+RobotDataWindowHeight);	
	glEnd();		
	text_->setTextColor(ViewerColor(1,1,1,1));
	text_->draw((int)(RobotDataWindowMinW+i*RobotDataWindowWidth/10), 
		    (int)(RobotDataWindowMinH+8+RobotDataWindowHeight),
		    "%.0f", (RobotDataTmin+i/10.*(RobotDataTmax-RobotDataTmin))/1000. );
    }
  }
} // namespace

// ---------------------------------------------------------------------------
// initRobotData
// ---------------------------------------------------------------------------
void initRobotData(int winId, Viewer3D* viewer) {
  if (!init_) {
    text_ = new ViewerText(10);
    winId_=winId;
    viewer_=viewer;
    glEnable(GL_BLEND);
    glShadeModel (GL_SMOOTH);
    glClearColor (0.0, 0.0, 0.0, 0.0);
    glClear (GL_COLOR_BUFFER_BIT);
    
#ifdef linux
    glutCreateMenu (menuRobotData);
    glutAttachMenu (GLUT_RIGHT_BUTTON);
#endif
    init_=true;
  }
  resetRobotData();
}


// ---------------------------------------------------------------------------
// robotDataSetTimeScale
// ---------------------------------------------------------------------------
void robotDataSetTimeScale(Millisecond Min, Millisecond Max)
{
  RobotDataTmin=Min;
  RobotDataTmax=Max;
}

// ---------------------------------------------------------------------------
// drawRobotDataLegend
// ---------------------------------------------------------------------------
void drawRobotDataLegend() 
{
  int i=0;
  int textX=(int)(RobotDataWindowMinW+6./10.*RobotDataWindowWidth); 
  int textY=(int)(RobotDataWindowMinH);
  text_->setTextSize(10);
   try {
    std::vector<ViewerData*>::iterator it=VIEWER3D->viewerData_.begin();
    for(;it!=VIEWER3D->viewerData_.end();it++) {
      if (!(*it)->display()) continue;
      (*it)->lock();
      text_->setTextColor((*it)->color());
      text_->draw(textX+15, textY+i*10,(*it)->name());
      glBegin(GL_LINES);
      glVertex2d(textX, textY-4+i*10);
      glVertex2d(textX+10, textY-4+i*10);
      glEnd();
      (*it)->unlock();
      i++;
    }
   } catch(...) {
   }
}

// ---------------------------------------------------------------------------
// drawRobotDataGraphs
// ---------------------------------------------------------------------------
void drawRobotDataGraphs()
{
  try {
    text_->setTextSize(8);
    std::vector<ViewerData*>::iterator it=VIEWER3D->viewerData_.begin();
    for(;it!=VIEWER3D->viewerData_.end();it++) {
      if (!(*it)->display()) continue;
      (*it)->lock();
      std::deque< std::pair<ViewerDataType, Millisecond> >::iterator it2=(*it)->list().begin();
      glColor3f((*it)->color().r, (*it)->color().g, (*it)->color().b);
      ViewerDataType deltay=(*it)->maxi()-(*it)->mini();
      if (deltay==0) deltay=1;
      Millisecond deltax=RobotDataTmax-RobotDataTmin;
      glBegin(GL_LINE_STRIP);
      for(;it2!=(*it)->list().end();it2++) {
	if ((*it2).second < RobotDataTmin) continue;
	if ((*it2).second > RobotDataTmax) break;
	glVertex2f(
		   RobotDataWindowMinW +(float)(((*it2).second-RobotDataTmin)*RobotDataWindowWidth/deltax),
		   RobotDataWindowHeight+RobotDataWindowMinH-(float)(((*it2).first-(*it)->mini())*RobotDataWindowHeight/deltay));
      }
      (*it)->unlock();
      glEnd();
    }
  } catch(...) {

  }
}

// ---------------------------------------------------------------------------
// resetRobotData
// ---------------------------------------------------------------------------
void resetRobotData()
{
#ifdef linux
    glutSetWindow(winId_);
#endif
    RobotDataTmin=0;
    RobotDataTmax=90000;
    /*   for (int i=lastMenuLength-1; i>=0;i--)
	 glutRemoveMenuItem(i);*/
    lastMenuLength=0;
}

// ---------------------------------------------------------------------------
// updateRobotDataMenu
// ---------------------------------------------------------------------------
void updateRobotDataMenu()
{
#ifdef linux
    glutSetWindow(winId_);
#endif
    /*
    if (lastMenuLength != VIEWER3D->viewerData_.size()) {
	//printf("%d %d\n", lastMenuLength, VIEWER3D->viewerData_.size());
	for(;lastMenuLength<VIEWER3D->viewerData_.size(); lastMenuLength++){
	    glutAddMenuEntry (VIEWER3D->viewerData_[lastMenuLength]->name(), 
			      lastMenuLength);
	}
    }
    */
}

// ---------------------------------------------------------------------------
// displayRobotData
// ---------------------------------------------------------------------------
void displayRobotData(void) {
    //Lock localLock(&viewer3DrepositoryLock);
    if (winId_==0) return;
    
#ifdef linux
    glutSetWindow(winId_);
#endif
    updateRobotDataMenu();
    drawGrid();
    drawRobotDataGraphs();
    drawRobotDataLegend();
    // draw
#ifdef linux
    glutSwapBuffers() ;
#endif
    Viewer3D::movieScreenShot(VIEWER_SCREEN_DATA);
    //fprintf(stdout, "Map\n");
}

// ---------------------------------------------------------------------------
// keyboardRobotData
// ---------------------------------------------------------------------------
void keyboardRobotData (unsigned char key, int x, int y){ 
  //printf("key:%d\n", key);
  Millisecond delta=RobotDataTmax-RobotDataTmin;
  switch (key) {
  case 43: //+
     RobotDataTmin+=delta/4;
     RobotDataTmax-=delta/4;
    break;
  case 53: //5
     RobotDataTmin=viewer_->time_-delta/4;
     RobotDataTmax=viewer_->time_+delta/4;
     if (RobotDataTmax>300000) {
       RobotDataTmax=300000;
       RobotDataTmin=RobotDataTmax-delta/2;
     }
     if (RobotDataTmin<0) {
       RobotDataTmin=0;
       RobotDataTmax=RobotDataTmin+delta/2;
     }
    break;
    case 42: //*
     RobotDataTmin=viewer_->time_-delta/2;
     RobotDataTmax=viewer_->time_+delta/2;
     if (RobotDataTmax>300000) {
       RobotDataTmax=300000;
       RobotDataTmin=RobotDataTmax-delta;
     }
     if (RobotDataTmin<0) {
       RobotDataTmin=0;
       RobotDataTmax=RobotDataTmin+delta;
     }
    break;
  case 45://-
    RobotDataTmin-=delta/2;
    RobotDataTmax+=delta/2;
    if (RobotDataTmin<0) RobotDataTmin=0;
    if (RobotDataTmax>300000) RobotDataTmax=300000;
    break;
  case 52://4
    RobotDataTmin-=delta/2;
    RobotDataTmax-=delta/2;
    if (RobotDataTmin<0) RobotDataTmin=0;
    RobotDataTmax=RobotDataTmin+delta;
    break;
  case 54://6
    RobotDataTmax+=delta/2;
    if (RobotDataTmax>300000) RobotDataTmax=300000;
    RobotDataTmin=RobotDataTmax-delta;
    break;
  case 'c':
      Viewer3D::screenShot("robotData");
      break;
  default:
    break;
  }
}


// ---------------------------------------------------------------------------
// menuRobotData
// ---------------------------------------------------------------------------
void menuRobotData (int value)
{
  VIEWER3D->viewerData_[value]->display() = ! VIEWER3D->viewerData_[value]->display();
}

// ---------------------------------------------------------------------------
// reshapeRobotData
// ---------------------------------------------------------------------------
void reshapeRobotData(int w, int h)
{
  RobotDataWindowHeight = (int)(h-2*RobotDataWindowMinH);
  RobotDataWindowWidth  = (int)(w-RobotDataWindowMinW);
  //Lock localLock(&viewer3DrepositoryLock);
  glViewport(0,0, w, h); 
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //gluOrtho2D(0.0, (GLfloat) 310, (GLfloat) 220);
  glOrtho(0.0f, w, h, 0.0f,-1.0f,1.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

// ---------------------------------------------------------------------------
// idleRobotData
// ---------------------------------------------------------------------------
void idleRobotData(void)
{
//		glutPostRedisplay();
}

// ---------------------------------------------------------------------------
// timerRobotData
// ---------------------------------------------------------------------------
void timerRobotData(int v)
{
 
}


