#include <stdio.h>
#include <math.h>
#include <assert.h>

#ifndef linux
#ifndef VIEWER_DISACTIVATED
#define VIEWER_DISACTIVATED
#endif
#endif
#ifndef VIEWER_DISACTIVATED 
#include <GL/glut.h>
#include "mthread.h"
#include "log.h"

#ifndef M_PI
#define M_PI (3.14)
#define M_PI_2 (M_PI/2.)
#define M_3_PI_2 (M_PI_2*3.)
#endif

#include "robotMap.h"
#include "robotView.h"
#include "robot3d.h"
#include "robotData.h"
#include "logPlayer.h"
#include "texture.h"
#include "gltools.h"
#endif // VIEWER_DISACTIVATED

#include "viewer3D.h"

// ---------------------------------------------------------------------------
// Viewer3D::drawRobot
// ---------------------------------------------------------------------------
void Viewer3D::drawRobot()
{
#ifndef VIEWER_DISACTIVATED 
    if (robotRP_.center.x<10) return;
    glPushMatrix ();
    glEnable(GL_DEPTH_TEST);
    glTranslatef(robotRP_.center.x*SCALE3D, robotRP_.center.y*SCALE3D, 0.0);
    glRotatef(r2d(robotRP_.direction), 0.0, 0.0 , 1.0);	
    glColor4f(1,1,1,1);
  
    texture_->apply(TEX_ROBOT_FRONT);
    glBegin(GL_QUADS);
    glTexCoord2f( 0.0, 0.0 ); glVertex3d(15, 15, 40);
    glTexCoord2f( 0.0, 1.0 ); glVertex3d(15, 15,  0);
    glTexCoord2f( 1.0, 1.0 ); glVertex3d(15, -15, 0);	
    glTexCoord2f( 1.0, 0.0 ); glVertex3d(15, -15,40);	
    glEnd();

    texture_->apply(TEX_ROBOT_BACK);
    glBegin(GL_QUADS);
    glTexCoord2f( 0.0, 0.0 ); glVertex3d(-15, 15, 40);
    glTexCoord2f( 0.0, 1.0 ); glVertex3d(-15, 15,  0);
    glTexCoord2f( 1.0, 1.0 ); glVertex3d(-15, -15, 0);	
    glTexCoord2f( 1.0, 0.0 ); glVertex3d(-15, -15,40);	
    glEnd();

    texture_->apply(TEX_ROBOT_SIDE_RIGHT);
    glBegin(GL_QUADS);
    glTexCoord2f( 1.0, 0.0 ); glVertex3d(-15, -15, 40);
    glTexCoord2f( 1.0, 1.0 ); glVertex3d(-15, -15, 0);
    glTexCoord2f( 0.0, 1.0 ); glVertex3d( 15, -15, 0);	
    glTexCoord2f( 0.0, 0.0 ); glVertex3d( 15, -15, 40);	
    glEnd(); 
    texture_->apply(TEX_ROBOT_SIDE_LEFT);
    glBegin(GL_QUADS);
    glTexCoord2f( 0.0, 0.0 ); glVertex3d( 15, 15, 40);
    glTexCoord2f( 0.0, 1.0 ); glVertex3d( 15, 15, 0);
    glTexCoord2f( 1.0, 1.0 ); glVertex3d(-15, 15, 0);	
    glTexCoord2f( 1.0, 0.0 ); glVertex3d(-15, 15, 40);	
    glEnd();
    texture_->apply(TEX_ROBOT_TOP);
    glBegin(GL_QUADS);
    glTexCoord2f( 0.0, 0.0 ); glVertex3d( 15, -15, 40);
    glTexCoord2f( 0.0, 1.0 ); glVertex3d(-15, -15, 40);
    glTexCoord2f( 1.0, 1.0 ); glVertex3d(-15, 15, 40);	
    glTexCoord2f( 1.0, 0.0 ); glVertex3d( 15, 15, 40);	
    glEnd(); 
    texture_->disable();

    if (VIEWER3D->armLeft_) {
        glBegin(GL_QUADS);
        glVertex3d(0, 30,34);	
	glVertex3d(0, 15,34);	
	glVertex3d(0, 15,36);	
	glVertex3d(0, 30,36);
        glEnd();
    }
    // armRight
    if (VIEWER3D->armRight_) {
        glBegin(GL_QUADS);
        glVertex3d(0, -30,34);	
	glVertex3d(0, -15,34);	
	glVertex3d(0, -15,36);	
	glVertex3d(0, -30,36);
        glEnd();
    }			// Done Drawing The Player
    
    // support de balise
    glColor4f(1,1,1,1);
    glPushMatrix();
    glTranslatef(-2.5,10.5,40);
    glBegin(GL_QUADS);
    glVertex3d(-4, -4,5);	
    glVertex3d(-4, 4,5);	
    glVertex3d(4, 4,5);	
    glVertex3d(4, -4,5);

    glVertex3d(-4, 0,0);	
    glVertex3d(-3.5, -0.5,0);	
    glVertex3d(-3.5, -0.5,5);	
    glVertex3d(-4, 0,5);
    glEnd();
    glPopMatrix();
    // gonio
    glPushMatrix();
    glTranslatef(-2.5,10.5,40);
    glColor4f(0.2,0.2,0.2,1);
    glRotatef(r2d(gonioDirection_-robotRP_.direction),0,0,1);	
    glBegin(GL_QUADS);
    glVertex3d(-3.5, 0,1);	
    glVertex3d(-3.5, 0,3);	
    glVertex3d(3.5, 0,3);	
    glVertex3d(3.5, 0,1);
    glColor4f(0.4,0.4,0.4,1);
    glVertex3d(0, -.5,0);	
    glVertex3d(0, -.5,3);	
    glVertex3d(0, .5,3);	
    glVertex3d(0, .5,0);
    glEnd();
    glPopMatrix();

    if (useTexture_) {
        // ombre
	//      glRotatef(-r2d(robotRP_.direction), 0.0, 0.0 , 1.0);
        glEnable(GL_ALPHA_TEST);
        glEnable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0,0,0,0.3);
      
        double x1, y1, x2, y2, dx, dy;
        dx = 6*cos(-robotRP_.direction+M_PI/4);
        dy = 6*sin(-robotRP_.direction+M_PI/4);
        x1=15; y1=15;
        x2=15; y2=-15;
        glBegin(GL_QUADS);
        glVertex3f(x1-dx, y1-dy, 1);
	glVertex3f(x2-dx, y2-dy, 1);
	glVertex3f(x2, y2, 1);
	glVertex3f(x1, y1, 1);
        glEnd();
      
        x1=x2; y1=y2;
        x2=-15; y2=-15;
        glBegin(GL_QUADS);
        glVertex3f(x1-dx, y1-dy, 1);
	glVertex3f(x2-dx, y2-dy, 1);
	glVertex3f(x2, y2, 1);
	glVertex3f(x1, y1, 1);
        glEnd();

        x1=x2; y1=y2;
        x2=-15; y2=15;
        glBegin(GL_QUADS);
        glVertex3f(x1-dx, y1-dy, 1);
	glVertex3f(x2-dx, y2-dy, 1);
	glVertex3f(x2, y2, 1);
	glVertex3f(x1, y1, 1);
        glEnd();

        x1=x2; y1=y2;
        x2=15; y2=15;
        glBegin(GL_QUADS);
        glVertex3f(x1-dx, y1-dy, 1);
	glVertex3f(x2-dx, y2-dy, 1);
	glVertex3f(x2, y2, 1);
	glVertex3f(x1, y1, 1);
        glEnd();

        if (VIEWER3D->armRight_) {
            x1=0; y1=-15;
            x2=0; y2=-30;
            glBegin(GL_QUADS);
            glVertex3f(x1-dx, y1-dy, 1);
            glVertex3f(x2-dx, y2-dy, 1);
            glVertex3f(x2-dx+1, y2-dy+1, 1);
            glVertex3f(x1-dx+1, y1-dy+1, 1);
            glEnd();
        }
        if (VIEWER3D->armLeft_) {
            x1=0; y1=15;
            x2=0; y2=30;
            glBegin(GL_QUADS);
            glVertex3f(x1-dx, y1-dy, 1);
            glVertex3f(x2-dx, y2-dy, 1);
            glVertex3f(x2-dx+1, y2-dy+1, 1);
            glVertex3f(x1-dx+1, y1-dy+1, 1);
            glEnd();
        }
        if (VIEWER3D->lowerArmOut_) {
            x1=15; y1=15;
            x2=25; y2=25;
            glBegin(GL_QUADS);
            glVertex3f(x1-dx, y1-dy, 1);
            glVertex3f(x2-dx, y2-dy, 1);
            glVertex3f(x2-dx+1, y2-dy+1, 1);
            glVertex3f(x1-dx+1, y1-dy+1, 1);
            glEnd();
        }
    }
    glPopMatrix ();
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawBG
// ---------------------------------------------------------------------------
void Viewer3D::drawBG()
{
#ifndef VIEWER_DISACTIVATED 
    // dessin rebord
    glPushMatrix ();
    glEnable(GL_DEPTH_TEST);
    int i,j;
    glColor3f(WHITE_BORDER);
    glBegin(GL_QUADS);	
    glVertex3d(0,0,7);
    glVertex3d(0,0,-20);	
    glVertex3d(300,0,-20);
    glVertex3d(300,0,7);	
    glEnd();
    glBegin(GL_QUADS);	
    glVertex3d(0,210,7);
    glVertex3d(0,210,-20);	
    glVertex3d(300,210,-20);
    glVertex3d(300,210,7);	
    glEnd();
    glBegin(GL_QUADS);	
    glVertex3d(0,210,7);
    glVertex3d(0,210,-20);	
    glVertex3d(0,0,-20);
    glVertex3d(0,0,7);	
    glEnd();
    glBegin(GL_QUADS);	
    glVertex3d(300,210,7);
    glVertex3d(300,210,-20);	
    glVertex3d(300,0,-20);
    glVertex3d(300,0,7);	
    glEnd();

    // dessin damier
    glColor3f(YELLOW_GRID_1);
    for(i=1;i<9;++i) {
        for(j=0;j<7;++j) {
            if ((i+j)%2==1) continue;
            glBegin(GL_QUADS);	
            glVertex3d(i*30,j*30,0);
            glVertex3d(i*30,(j+1)*30,0);	
            glVertex3d((i+1)*30,(j+1)*30,0);
            glVertex3d((i+1)*30,j*30,0);	
            glEnd();			
        }
    }
    glColor3f(YELLOW_GRID_2);
    for(i=1;i<9;++i) {
        for(j=0;j<7;++j) {
            if ((i+j)%2==0) continue;
            glBegin(GL_QUADS);
            glVertex3d(i*30,j*30,0);
            glVertex3d(i*30,(j+1)*30,0);
            glVertex3d((i+1)*30,(j+1)*30,0);
            glVertex3d((i+1)*30,j*30,0);	
            glEnd();		
        }
    }

    // dessin de la zone d'en but
    glColor3f(BLUE_BORDER);
    glBegin(GL_QUADS);
    glVertex3d(0, 0, 0);
    glVertex3d(0, 210, 0);
    glVertex3d(30, 210, 0);
    glVertex3d(30, 0, 0);

    glVertex3d(270, 0, 0);
    glVertex3d(270, 210, 0);
    glVertex3d(300, 210, 0);
    glVertex3d(300, 0, 0);	
    glEnd();

    // dessin de la bordure de la zone d'en but
    glColor3f(BLUE_BORDER_2);
    glBegin(GL_QUADS);
    glVertex3d(28, 0, 2);
    glVertex3d(28, 75, 2);
    glVertex3d(30, 75, 2);
    glVertex3d(30, 0, 2);

    glVertex3d(30, 0, 0);
    glVertex3d(30, 75, 0);
    glVertex3d(30, 75, 2);
    glVertex3d(30, 0, 2);

    glVertex3d(28, 210, 2);
    glVertex3d(28, 135, 2);
    glVertex3d(30, 135, 2);
    glVertex3d(30, 210, 2);

    glVertex3d(30, 210, 0);
    glVertex3d(30, 135, 0);
    glVertex3d(30, 135, 2);
    glVertex3d(30, 210, 2);

    glVertex3d(272, 0, 2);
    glVertex3d(272, 75, 2);
    glVertex3d(270, 75, 2);
    glVertex3d(270, 0, 2);

    glVertex3d(270, 0, 0);
    glVertex3d(270, 75, 0);
    glVertex3d(270, 75, 2);
    glVertex3d(270, 0, 2);

    glVertex3d(272, 210, 2);
    glVertex3d(272, 135, 2);
    glVertex3d(270, 135, 2);
    glVertex3d(270, 210, 2);

    glVertex3d(270, 210, 0);
    glVertex3d(270, 135, 0);
    glVertex3d(270, 135, 2);
    glVertex3d(270, 210, 2);
    glEnd();

    // dessin des pilliers horizontaux
    glColor3f(GREEN_COCONUT);
    glBegin(GL_QUADS);
    glVertex3d(29, 135, 38);
    glVertex3d(29, 75, 38);
    glVertex3d(29, 75, 40);
    glVertex3d(29, 135, 40);

    glVertex3d(28, 135, 39);
    glVertex3d(28, 75, 39);
    glVertex3d(30, 75, 39);
    glVertex3d(30, 135, 39);

    glVertex3d(29, 135, 88);
    glVertex3d(29, 75,  88);
    glVertex3d(29, 75,  90);
    glVertex3d(29, 135, 90);

    glVertex3d(28, 135, 89);
    glVertex3d(28, 75, 89);
    glVertex3d(30, 75, 89);
    glVertex3d(30, 135, 89);
    glEnd();

    drawPalmTree(300, 1350);
    drawPalmTree(300, 750);

    glColor3f(GREEN_COCONUT);
    glBegin(GL_QUADS);
    glVertex3d(271, 135, 38);
    glVertex3d(271, 75, 38);
    glVertex3d(271, 75, 40);
    glVertex3d(271, 135, 40);

    glVertex3d(270, 135, 39);
    glVertex3d(270, 75, 39);
    glVertex3d(272, 75, 39);
    glVertex3d(272, 135, 39);
  
    glVertex3d(271, 135, 88);
    glVertex3d(271, 75, 88);
    glVertex3d(271, 75, 90);
    glVertex3d(271, 135, 90);

    glVertex3d(270, 135, 89);
    glVertex3d(270, 75, 89);
    glVertex3d(272, 75, 89);
    glVertex3d(272, 135, 89);
    glEnd();

    drawPalmTree(2700, 1350);
    drawPalmTree(2700, 750);

    // dessin des supports de balise 
    glColor3f(GREEN_COCONUT);
    glBegin(GL_QUADS);
    glVertex3d(34, 0, 45);
    glVertex3d(26, 0, 45);
    glVertex3d(26, -8, 45);
    glVertex3d(34, -8, 45);

    glVertex3d(30, 0, 45);
    glVertex3d(30, 0, 0);
    glVertex3d(30, -2, 0);
    glVertex3d(30, -2, 45);

    glVertex3d(34, 75, 45);
    glVertex3d(26, 75, 45);
    glVertex3d(26, 67, 45);
    glVertex3d(34, 67, 45);

    glVertex3d(34, 135, 45);
    glVertex3d(26, 135, 45);
    glVertex3d(26, 143, 45);
    glVertex3d(34, 143, 45);

    glVertex3d(34, 210, 45);
    glVertex3d(26, 210, 45);
    glVertex3d(26, 218, 45);
    glVertex3d(34, 218, 45);

    glVertex3d(30, 210, 45);
    glVertex3d(30, 210, 0);
    glVertex3d(30, 212, 0);
    glVertex3d(30, 212, 45);

    glEnd();

    // dessin des supports de balise 
    glColor3f(GREEN_COCONUT);
    glBegin(GL_QUADS);
    glVertex3d(274, 0, 45);
    glVertex3d(266, 0, 45);
    glVertex3d(266, -8, 45);
    glVertex3d(274, -8, 45);

    glVertex3d(270, 0, 45);
    glVertex3d(270, 0, 0);
    glVertex3d(270, -2, 0);
    glVertex3d(270, -2, 45);

    glVertex3d(274, 75, 45);
    glVertex3d(266, 75, 45);
    glVertex3d(266, 67, 45);
    glVertex3d(274, 67, 45);

    glVertex3d(274, 135, 45);
    glVertex3d(266, 135, 45);
    glVertex3d(266, 143, 45);
    glVertex3d(274, 143, 45);

    glVertex3d(274, 210, 45);
    glVertex3d(266, 210, 45);
    glVertex3d(266, 218, 45);
    glVertex3d(274, 218, 45);

    glVertex3d(270, 210, 45);
    glVertex3d(270, 210, 0);
    glVertex3d(270, 212, 0);
    glVertex3d(270, 212, 45);

    glEnd();

    if (useTexture_) {  
        glColor4f(1,1,1,1);
        texture_->apply(TEX_LOGO);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3f(160, -1,  5);
        glTexCoord2f( 0.0, 1.0 ); glVertex3f(160, -1, -10);
        glTexCoord2f( 1.0, 1.0 ); glVertex3f(140, -1, -10);
        glTexCoord2f( 1.0, 0.0 ); glVertex3f(140, -1,  5);		
        glEnd();
        texture_->disable();
  
        texture_->apply(TEX_SALLE_SIDE3);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(-100, 350,  200);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(-100, 350,  -100);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(-100, -200, -100);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(-100, -200, 200);	
        glEnd(); 

        texture_->apply(TEX_SALLE_SIDE1);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(400, -200, 200);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(400, -200, -100);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(400, 350,  -100);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(400, 350,  200);	
        glEnd(); 

        texture_->apply(TEX_SALLE_SIDE4);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(400,  350, 200);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(400,  350, -100);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(-100, 350,  -100);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(-100, 350,  200);	
        glEnd();

        texture_->apply(TEX_SALLE_SIDE2);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(-100,  -200, 200);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(-100,  -200, -100);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(400, -200, -100);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(400, -200, 200);	
        glEnd(); 
        texture_->disable(); 

        // texture_->apply(TEX_SALLE_SOL);
        glColor4f(0.43,0.33,0.24,1);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(-100,  350, -100);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(-100,  -200, -100);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(400, -200, -100);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(400, 350, -100);	
        glEnd();

        glColor4f(0.40,0.30,0.20,1);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(-10,  200, -99);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(-10,  -10, -99);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(290, -10, -99);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(290, 200, -99);	
        glEnd(); 
        // texture_->apply(TEX_SALLE_PLAFOND);
        glColor4f(1,1,1,1);
        glBegin(GL_QUADS);
        glTexCoord2f( 0.0, 0.0 ); glVertex3d(-100,  350, 200);
        glTexCoord2f( 0.0, 1.0 ); glVertex3d(-100,  -200, 200);
        glTexCoord2f( 1.0, 1.0 ); glVertex3d(400, -200, 200);	
        glTexCoord2f( 1.0, 0.0 ); glVertex3d(400, 350, 200);	
        glEnd(); 


        // pieds de la table
        glColor4f(1,1,1,1);
        glBegin(GL_QUADS);
        glVertex3d(10, 12, -2);
        glVertex3d(15, 12, -2);
        glVertex3d(15, 12, -100);	
        glVertex3d(10, 12, -100);	

        glVertex3d(12, 10, -2);
        glVertex3d(12, 15, -2);
        glVertex3d(12, 15, -100);	
        glVertex3d(12, 10, -100);

        glVertex3d(10, 202, -2);
        glVertex3d(15, 202, -2);
        glVertex3d(15, 202, -100);	
        glVertex3d(10, 202, -100);	

        glVertex3d(12, 200, -2);
        glVertex3d(12, 205, -2);
        glVertex3d(12, 205, -100);	
        glVertex3d(12, 200, -100);


        glVertex3d(150, 12, -2);
        glVertex3d(155, 12, -2);
        glVertex3d(155, 12, -100);	
        glVertex3d(150, 12, -100);	

        glVertex3d(152, 10, -2);
        glVertex3d(152, 15, -2);
        glVertex3d(152, 15, -100);	
        glVertex3d(152, 10, -100);

        glVertex3d(150, 202, -2);
        glVertex3d(155, 202, -2);
        glVertex3d(155, 202, -100);	
        glVertex3d(150, 202, -100);	

        glVertex3d(152, 200, -2);
        glVertex3d(152, 205, -2);
        glVertex3d(152, 205, -100);	
        glVertex3d(152, 200, -100);


        glVertex3d(290, 12, -2);
        glVertex3d(295, 12, -2);
        glVertex3d(295, 12, -100);	
        glVertex3d(290, 12, -100);	

        glVertex3d(292, 10, -2);
        glVertex3d(292, 15, -2);
        glVertex3d(292, 15, -100);	
        glVertex3d(292, 10, -100);

        glVertex3d(290, 202, -2);
        glVertex3d(295, 202, -2);
        glVertex3d(295, 202, -100);	
        glVertex3d(290, 202, -100);	

        glVertex3d(292, 200, -2);
        glVertex3d(292, 205, -2);
        glVertex3d(292, 205, -100);	
        glVertex3d(292, 200, -100);	
        glEnd(); 
    }

    glPopMatrix ();
#endif // VIEWER_DISACTIVATED 
};


void Viewer3D::drawFillets()
{
#ifndef VIEWER_DISACTIVATED 
    glPushMatrix ();
    if(useTexture_) {
        // fillets
        // dessin des pilliers horizontaux
        glColor3f(GREEN_COCONUT);
        glBegin(GL_QUADS);
        glVertex3d(30, 75, 39);
        glVertex3d(30, 75, 40);
        glVertex3d(0, 75, 40);
        glVertex3d(0, 75, 39);

        glVertex3d(30, 135, 39);
        glVertex3d(30, 135, 40);
        glVertex3d(0, 135, 40);
        glVertex3d(0, 135, 39);

        glVertex3d(300, 75, 39);
        glVertex3d(300, 75, 40);
        glVertex3d(270, 75, 40);
        glVertex3d(270, 75, 39);

        glVertex3d(300, 135, 39);
        glVertex3d(300, 135, 40);
        glVertex3d(270, 135, 40);
        glVertex3d(270, 135, 39);
        glEnd();

        glEnable(GL_ALPHA_TEST);
        glEnable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0.5,0.5,0.5,0.5);
        glBegin(GL_QUADS);
        glVertex3d(30, 75, 90);
        glVertex3d(30, 135, 90);
        glVertex3d(0, 135, 38);	
        glVertex3d(0, 75, 38);
    
        glVertex3d(30, 75, 40);
        glVertex3d(30, 135, 40);
        glVertex3d(0, 135, 38);	
        glVertex3d(0, 75, 38);
        glEnd();
        glBegin(GL_TRIANGLES);
        glVertex3d(30, 75, 90);
        glVertex3d(30, 75, 40);
        glVertex3d(0, 75, 38);

        glVertex3d(30, 135, 90);
        glVertex3d(30, 135, 40);
        glVertex3d(0, 135, 38);
        glEnd();

        glBegin(GL_QUADS);
        glVertex3d(270, 75, 90);
        glVertex3d(270, 135, 90);
        glVertex3d(300, 135, 38);	
        glVertex3d(300, 75, 38);

        glVertex3d(270, 75, 40);
        glVertex3d(270, 135, 40);
        glVertex3d(300, 135, 38);	
        glVertex3d(300, 75, 38);
        glEnd();
        glBegin(GL_TRIANGLES);
        glVertex3d(270, 75, 90);
        glVertex3d(270, 75, 40);
        glVertex3d(300, 75, 38);

        glVertex3d(270, 135, 90);
        glVertex3d(270, 135, 40);
        glVertex3d(300, 135, 38);
        glEnd();

        // ombre
        glColor4f(0.2,0.2,0.2,0.3);
        glBegin(GL_QUADS);
        glVertex3d(27, 60, 1);
        glVertex3d(27, 130, 1);
        glVertex3d(5, 130, 1);	
        glVertex3d(5, 70, 1);

        glVertex3d(297, 70, 1);
        glVertex3d(297, 130, 1);
        glVertex3d(265, 130, 1);	
        glVertex3d(265, 60, 1);
        glEnd();

        glDisable(GL_ALPHA_TEST);
    }

    glPopMatrix ();
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawEnnemy
// ---------------------------------------------------------------------------
void Viewer3D::drawEnnemy()
{
#ifndef VIEWER_DISACTIVATED 
    if (ennemyRP_.center.x<10) return;
    GLUquadric* quad=gluNewQuadric();
    glPushMatrix ();
    glColor4f(ENEMY_RP, 1.0f);
    glTranslatef(ennemyRP_.center.x*SCALE3D, 
		 ennemyRP_.center.y*SCALE3D, 
		 0);
    gluCylinder(quad, ENNEMY_RP_RAYON*SCALE3D, ENNEMY_RP_RAYON*SCALE3D, 
		400*SCALE3D, 16, 1);
    glTranslatef(0, 0, 400*SCALE3D);
    gluDisk(quad, 0, ENNEMY_RP_RAYON*SCALE3D, 12,12);
    glPopMatrix();
    gluDeleteQuadric(quad);
#endif
}

// ---------------------------------------------------------------------------
// Viewer3D::drawEnnemyRS
// ---------------------------------------------------------------------------
void Viewer3D::drawEnnemyRS()
{
#ifndef VIEWER_DISACTIVATED 
    if (ennemyRS_.center.x<10) return;
    glPushMatrix ();
    glColor4f(ENEMY_RS, 1.0f);
    glTranslatef(ennemyRS_.center.x*SCALE3D, 
		 ennemyRS_.center.y*SCALE3D, 
		 0);
    glRotatef(r2d(ennemyRS_.direction), 
	      0, 
	      0,
	      1);
    glScalef(SCALE3D, SCALE3D, 1);
    glBegin(GL_QUADS);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA),  20);

    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI),  20);

    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI),  20);

    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI), 0);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA),  20);

    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI), 20);
    glVertex3d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI), 
               ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI),  20);

    glEnd();
    glPopMatrix();
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawRobotRS
// ---------------------------------------------------------------------------
void Viewer3D::drawRobotRS()
{
#ifndef VIEWER_DISACTIVATED 
    if (robotRS_.center.x<10) return;
    glPushMatrix ();
    glColor4f(ROBOT_RS, 1.0f);
    glTranslatef(robotRS_.center.x*SCALE3D, 
		 robotRS_.center.y*SCALE3D, 
		 0);
    GLUquadric* quad=gluNewQuadric();

    gluCylinder(quad, ROBOT_RS_RAYON*SCALE3D, ROBOT_RS_RAYON*SCALE3D, 
		40, 16, 1);
    glTranslatef(0, 0, 200*SCALE3D);
    gluDisk(quad, 0, ROBOT_RS_RAYON*SCALE3D, 12,12);
        
    gluDeleteQuadric(quad);
    glPopMatrix();
    
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawPalmTree
// ---------------------------------------------------------------------------
void Viewer3D::drawPalmTree(Millimeter x, Millimeter y, bool middle)
{
#ifndef VIEWER_DISACTIVATED 
    GLUquadric* quad=gluNewQuadric();
    glPushMatrix ();
    glColor4f(GREEN_COCONUT, 1.0f);
    glTranslatef(x*SCALE3D, 
		 y*SCALE3D, 
		 0);
    gluCylinder(quad, 14*SCALE3D, 14*SCALE3D, 
		middle?60:100, 9, 1);
    glTranslatef(0, 
		 0, 
		 middle?60:100);
    for(double theta=0; theta<2*M_PI;theta+=M_PI/2.5) {
        glBegin(GL_QUADS);
        glVertex3f(0, 0, 0);
	glVertex3f(8*cos(theta-M_PI/6), 8*sin(theta-M_PI/6), -3);
	glVertex3f(20*cos(theta), 20*sin(theta),-8);
	glVertex3f(8*cos(theta+M_PI/6), 8*sin(theta+M_PI/6),-3);
	glEnd();
    } 
    // ombre
    if (useTexture_) {
        glTranslatef(0, 
                     0, 
                     middle?-60:-100);
        glEnable(GL_ALPHA_TEST);
        glEnable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0,0,0,0.3);
        glBegin(GL_QUADS);
        glVertex3f(2, -2, 1);
	glVertex3f(-2, 2, 1);
	glVertex3f(middle?-7:-11, middle?-5:-9,1);
	glVertex3f(middle?-5:-9, middle?-7:-11,1);
        glEnd();
        for(double theta=0; theta<2*M_PI;theta+=M_PI/2.5) {
            glBegin(GL_QUADS);
            glVertex3f((middle?-6:-10)+0, (middle?-6:-10)+0, 1);
            glVertex3f((middle?-6:-10)+8*cos(theta-M_PI/6), (middle?-6:-10)+8*sin(theta-M_PI/6), 1);
            glVertex3f((middle?-6:-10)+20*cos(theta), (middle?-6:-10)+20*sin(theta),1);
            glVertex3f((middle?-6:-10)+8*cos(theta+M_PI/6), (middle?-6:-10)+8*sin(theta+M_PI/6),1);
            glEnd();
        }	;
        glDisable(GL_ALPHA_TEST);
    }
    glPopMatrix();
    gluDeleteQuadric(quad);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawPalmTrees
// ---------------------------------------------------------------------------
void Viewer3D::drawPalmTrees()
{
#ifndef VIEWER_DISACTIVATED 
    drawPalmTree(coconut_.x, coconut_.y, true);
    drawPalmTree(TERRAIN_LONGUEUR - coconut_.x, 
                 TERRAIN_LARGEUR - coconut_.y, true);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawBall
// ---------------------------------------------------------------------------
void Viewer3D::drawBall(Millimeter x, Millimeter y, Millimeter z)
{
#ifndef VIEWER_DISACTIVATED 
    GLUquadric* quad=gluNewQuadric();
    glPushMatrix ();
    glColor4f(RED_BALL, 1.0f);
    glTranslatef(x*SCALE3D, 
		 y*SCALE3D, 
		 z*SCALE3D);
    glScalef(1,1.5,1); // pour faire l'oval
    gluSphere(quad, BALLON_RAYON*SCALE3D, 9,4);
    if (useTexture_) {
        glTranslatef(-z*SCALE3D/10,-z*SCALE3D/10,-z*SCALE3D+0.5);
        glEnable(GL_ALPHA_TEST);
        glEnable(GL_DEPTH_TEST);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glColor4f(0,0,0,0.3); 
        gluDisk (quad, 0, BALLON_RAYON*SCALE3D, 9, 4);
    }
    glPopMatrix();
    gluDeleteQuadric(quad);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::drawBalls
// ---------------------------------------------------------------------------
void Viewer3D::drawBalls()
{
#ifndef VIEWER_DISACTIVATED 
    for(unsigned int i=0;i<BALLON_NBR;i++) {
        if (!balls_ || balls_[i].inRobot) continue;
        drawBall(balls_[i].center.x, balls_[i].center.y, balls_[i].altitude);
    }
#endif
}

// ---------------------------------------------------------------------------
// Viewer3D::drawObstacles
// ---------------------------------------------------------------------------
void Viewer3D::drawObstacles()
{
#ifndef VIEWER_DISACTIVATED 
    glEnable(GL_ALPHA_TEST);
    glEnable(GL_DEPTH_TEST);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glColor4f(1.0f,0.0f,0.0f, 0.5f);
    ListOfObstacles::iterator it=obstacles_[0].begin();
    GLUquadric* quad=gluNewQuadric();
    for(;it!=obstacles_[0].end() && quad; it++) {
        glPushMatrix ();
        glColor4f(1.0f,0.0f,0.0f, 0.5f);
        glTranslatef((*it).center.x*SCALE3D, (*it).center.y*SCALE3D, 50.0*SCALE3D);
        gluSphere(quad, (*it).radius*SCALE3D, 10,5);
        glPopMatrix();

        glPushMatrix ();
        glColor4f(1.0f,0.0f,0.0f, 1.0f);
        glTranslatef((*it).detectionPoint.x*SCALE3D, 
                     (*it).detectionPoint.y*SCALE3D, 
                     10.0*SCALE3D);
        gluSphere(quad, 30*SCALE3D, 50,5);
        glPopMatrix();
    }
    gluDeleteQuadric(quad);
    //glDisable(GL_DEPTH_TEST);
    glDisable(GL_ALPHA_TEST);
#endif // VIEWER_DISACTIVATED 
}
