#include <stdio.h>
#ifdef linux
#include <GL/glut.h>
#endif

#include "robotMap.h"
#include "gltools.h"
#include "viewer3D.h"
#include "robotConst.h"
#include "mthread.h"
#include "simulator.h"

#include "text.h"

extern pthread_mutex_t viewer3DrepositoryLock;

#define COLOR_DETECTED_BALL    1.0f,0.0f,1.0f, 0.5f
#define COLOR_ASSERVISSEMENT_BALL    1.0f,1.0f,0.0f, 0.5f
#define COLOR_DETECTED_COCONUT 1.0f,0.0f,0.0f, 0.5f
#define COLOR_BUMPER_OBSTACLE  0.0f,0.0f,1.0f, 0.5f
#define COLOR_ENV_OBSTACLE     0.5f,0.0f,1.0f, 0.5f
#define COLOR_USER_OBSTACLE    1.0f,0.0f,0.7f, 0.5f
#define COLOR_TRAJEC_OBSTACLE  0.5f,0.5f,0.5f, 0.5f 

#define COLOR_GRID_BALL        1.0f, 0.0f, 0.0f, 0.5f
#define COLOR_GRID_BALL_SYMETRIE 0.8f, 0.0f, 0.0f, 0.3f
#define COLOR_GRID_OBSTACLE    0.0f, 0.0f, 1.0f, 0.5f
#define COLOR_GRID_VISITED     1.0f, 1.0f, 0.0f, 0.5f
#define COLOR_GRID_COCONUT     0.0f, 1.0f, 0.0f, 0.5f

static const int MAP_MAX_BALL_DISPLAYED =15;
// --------------------------------------------------------------------
// private namespace
// --------------------------------------------------------------------
namespace {

    Viewer3D* viewer_=NULL;
    int winId_=0;
    bool init_=false;
    
    double mouseScaleX_=1, mouseScaleY_=1;
    
    ViewerText *text_;
    
    static const double SCALE =0.1;
    static const int    MARGIN=5;
    
    static const int COVERED_X=60;
    static const int COVERED_Y=42;
    static const int COVERED_SCALE_X=(int)(SCALE*TERRAIN_X/(double)COVERED_X);
    static const int COVERED_SCALE_Y=(int)(SCALE*TERRAIN_Y/(double)COVERED_Y);
    
    Trajectory trajectory_;
    Trajectory targetTrajectory_;
    int coveredZone_[COVERED_X][COVERED_Y];
    int date_=0;
    
    bool displayCovered_=false;
    double coveredDelay_=1000.;
    bool displayTrajectory_=true;
    bool displayCurrentTrajectory_=true;
    bool displayGrid_=true;
    // --------------------------------------------------------------------
    // drawBG
    // --------------------------------------------------------------------
    void drawBG() {
        glLoadIdentity();
        glClear (GL_COLOR_BUFFER_BIT);// Reset The Modelview Matrix
        glClear (GL_DEPTH_BUFFER_BIT);// Reset The Modelview Matrix
        int i,j;

        // dessin du contour
        glColor3f(1.0f,1.0f,1.0f);
        glBegin(GL_LINE_STRIP);
        glVertex2d(MARGIN, MARGIN);	
        glVertex2d(MARGIN, MARGIN+TERRAIN_Y*SCALE);	
        glVertex2d(MARGIN+TERRAIN_X*SCALE, MARGIN+TERRAIN_Y*SCALE);
        glVertex2d(MARGIN+TERRAIN_X*SCALE, MARGIN);
        glVertex2d(MARGIN, MARGIN);
        glEnd();	
	
        //dessin de la grille
        for(i=1;i<9;++i) {
            for(j=0;j<7;++j) {
                if ((i+j)%2==0) {
                    glColor3f(YELLOW_GRID_1);
                } else {
                    glColor3f(YELLOW_GRID_2);
                }
                glBegin(GL_QUADS);
                glVertex2d(MARGIN+i*30,MARGIN+j*30);	
                glVertex2d(MARGIN+i*30,MARGIN+(j+1)*30);	
                glVertex2d(MARGIN+(i+1)*30,MARGIN+(j+1)*30);
                glVertex2d(MARGIN+(i+1)*30,MARGIN+j*30);	
                glEnd();		
            }
        }

        // dessin de la zone d'en but
        glColor3f(BLUE_BORDER);
        glBegin(GL_QUADS);
        glVertex2d(MARGIN,MARGIN);	
        glVertex2d(MARGIN,MARGIN+210);	
        glVertex2d(MARGIN+30,MARGIN+210);
        glVertex2d(MARGIN+30,MARGIN);

        glVertex2d(MARGIN+270,MARGIN);	
        glVertex2d(MARGIN+270,MARGIN+210);	
        glVertex2d(MARGIN+300,MARGIN+210);
        glVertex2d(MARGIN+300,MARGIN);
        glEnd();

        glColor3f(BLUE_BORDER_2);
        glBegin(GL_QUADS);
        glVertex2d(MARGIN+29,MARGIN);	
        glVertex2d(MARGIN+29,MARGIN+210);	
        glVertex2d(MARGIN+31,MARGIN+210);
        glVertex2d(MARGIN+31,MARGIN);

        glVertex2d(MARGIN+269,MARGIN);	
        glVertex2d(MARGIN+269,MARGIN+210);	
        glVertex2d(MARGIN+271,MARGIN+210);
        glVertex2d(MARGIN+271,MARGIN);
        glEnd();

        // buts
        glColor3f(GREEN_COCONUT);
        glBegin(GL_QUADS);
        glVertex2d(MARGIN+29,MARGIN+75);	
        glVertex2d(MARGIN+29,MARGIN+135);	
        glVertex2d(MARGIN+31,MARGIN+135);
        glVertex2d(MARGIN+31,MARGIN+75);

        glVertex2d(MARGIN+269,MARGIN+75);	
        glVertex2d(MARGIN+269,MARGIN+135);	
        glVertex2d(MARGIN+271,MARGIN+135);
        glVertex2d(MARGIN+271,MARGIN+75);
        glEnd();

        glDisable(GL_DEPTH_TEST);

    }

    // --------------------------------------------------------------------
    // drawPalmTree
    // --------------------------------------------------------------------
    void drawPalmTree(Millimeter x, Millimeter y, bool real=true) {
        GLUquadric* quad=gluNewQuadric();

        //glDisable(GL_DEPTH_TEST);
        glPushMatrix ();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+(TERRAIN_Y-y)*SCALE,0); 
	if (!real) {
	    glColor4f(COLOR_DETECTED_COCONUT);
	} else {
	    glColor4f(GREEN_COCONUT, 0.5f);
	}
        for(double theta=0; theta<2*M_PI;theta+=M_PI/2.5) {
            glBegin(GL_QUADS);
            glVertex2d(0, 0);
            glVertex2d(8*cos(theta-M_PI/6), 8*sin(theta-M_PI/6));
            glVertex2d(20*cos(theta), 20*sin(theta));
            glVertex2d(8*cos(theta+M_PI/6), 8*sin(theta+M_PI/6));
            glEnd();
        }	
        //glDisable(GL_ALPHA_TEST);

        glColor4f(GREEN_COCONUT, 1.0f);
        gluDisk(quad, 0, 1.4*SCALE, 5,5);
        glPopMatrix();
    
        gluDeleteQuadric(quad);
    }

    // --------------------------------------------------------------------
    // drawBall
    // --------------------------------------------------------------------
    void drawBall(Millimeter x, Millimeter y) {
        GLUquadric* quad=gluNewQuadric();
        glPushMatrix ();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+(TERRAIN_Y-y)*SCALE,0); 
	glScalef(1.,1.5,1.);
        glColor4f(RED_BALL, 1.0f);
        gluDisk(quad, 0, 30*SCALE, 5,5);
        glPopMatrix();
    
        gluDeleteQuadric(quad);
    }

    // --------------------------------------------------------------------
    // drawEnnemy
    // --------------------------------------------------------------------
    void drawEnnemy(Millimeter x, Millimeter y, Radian t) {
	if (x<10) return;
        GLUquadric* quad=gluNewQuadric();
        glPushMatrix ();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+(TERRAIN_Y-y)*SCALE,0); 
        glRotatef(r2d(t),0,0,1); 
        glColor4f(ENEMY_RP, 1.0f);
        gluDisk(quad, 0, ENNEMY_RP_RAYON*SCALE, 12,12);
        glPopMatrix();
    
        gluDeleteQuadric(quad);
    }

    // --------------------------------------------------------------------
    // drawEnnemyRS
    // --------------------------------------------------------------------
    void drawEnnemyRS(Millimeter x, Millimeter y, Radian t) {
	if (x<10) return;
        glPushMatrix ();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+(TERRAIN_Y-y)*SCALE,0); 
        glRotatef(r2d(t),0,0,1);
        glColor4f(ENEMY_RS, 1.0f); 
        glBegin(GL_QUADS);
        glVertex2d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA)*SCALE, 
                   ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA)*SCALE);
        glVertex2d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA)*SCALE, 
                   ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA)*SCALE);
        glVertex2d(ENNEMY_RS_RAYON*cos(ENNEMY_RS_THETA+M_PI)*SCALE, 
                   ENNEMY_RS_RAYON*sin(ENNEMY_RS_THETA+M_PI)*SCALE);
        glVertex2d(ENNEMY_RS_RAYON*cos(-ENNEMY_RS_THETA+M_PI)*SCALE, 
                   ENNEMY_RS_RAYON*sin(-ENNEMY_RS_THETA+M_PI)*SCALE);
        glEnd();
        glPopMatrix();
    }

    // --------------------------------------------------------------------
    // drawRobotRS
    // --------------------------------------------------------------------
    void drawRobotRS(Millimeter x, Millimeter y, Radian t) {
	if (x<10) return;
        glPushMatrix ();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+(TERRAIN_Y-y)*SCALE,0);
	GLUquadric* quad=gluNewQuadric();
        glRotatef(r2d(t),0,0,1); 
        glColor4f(ROBOT_RS, 1.0f);
        gluDisk(quad, 0, ROBOT_RS_RAYON*SCALE, 12,12);
        glPopMatrix();
    }
    // --------------------------------------------------------------------
    // setCoveredZone
    // --------------------------------------------------------------------
    void setCoveredZone(Millimeter x, Millimeter y, Radian t) {
        int i, j;
        int xi, xj;
        for(i=-2;i<=2;i++) {
            for(j=-2;j<=2;++j) {
                xi=(int)(x/COVERED_SCALE_X*SCALE)+i;
                xj=(int)(y/COVERED_SCALE_Y*SCALE)+j;
                if (xi<0 || xi>=COVERED_X) continue;
                if (xj<0 || xj>=COVERED_Y) continue;
                coveredZone_[xi][xj]=max(viewer_->time_,0);
            }
        }
    }

    // --------------------------------------------------------------------
    // drawGrid
    // --------------------------------------------------------------------
    void drawGrid(LogGrid* grid) {
	if (grid == NULL || !displayGrid_) return;
	int i,j;
	for(i=0;i<grid->sizeX;i++) {
            for(j=0;j<grid->sizeY;j++) {
		switch(grid->elements[i*grid->sizeY+j]) {
		case GRID_BALL:
		    glColor4f(COLOR_GRID_BALL);
		    break;
		case GRID_BALL_SYMETRIE:
		    glColor4f(COLOR_GRID_BALL_SYMETRIE);
		    break;
		case GRID_OBSTACLE:
		    glColor4f(COLOR_GRID_OBSTACLE);
		    break;
		case GRID_VISITED:
		    glColor4f(COLOR_GRID_VISITED);
		    break;
		case GRID_COCONUT:
		    glColor4f(COLOR_GRID_COCONUT);
		    break;
		case GRID_UNKNOWN:
		default:
		    continue;
		} 
		glPushMatrix();
		glTranslatef(MARGIN+(i+1)*300*SCALE, 
                             MARGIN+TERRAIN_Y*SCALE-(j+1)*300*SCALE,0);
                glBegin(GL_QUADS);
		glVertex2d(150*SCALE,   150*SCALE);
                glVertex2d(-150*SCALE,  150*SCALE);
                glVertex2d(-150*SCALE, -150*SCALE);
                glVertex2d(150*SCALE,  -150*SCALE);
		glEnd();
		glPopMatrix();
                	
	    }
	}
    }

    // --------------------------------------------------------------------
    // drawCoverZone
    // --------------------------------------------------------------------
    void drawCoverZone() {
        if (!displayCovered_) return;
        //glEnable(GL_ALPHA_TEST);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glBegin(GL_QUADS);
        int i,j;
        for(i=0;i<COVERED_X;i++) {
            for(j=0;j<COVERED_Y;j++) {
                if (coveredZone_[i][j] == 0 || coveredZone_[i][j]<viewer_->time_-coveredDelay_) continue;
                glColor4f(0.0f,1.0f,0.0f, 0.7*(1.+(coveredZone_[i][j]-viewer_->time_)/coveredDelay_));
                glVertex2d(MARGIN+ (i-0.5)*COVERED_SCALE_X, 
                           TERRAIN_Y*SCALE+MARGIN-(j-0.5)*COVERED_SCALE_Y);
                glVertex2d(MARGIN+ (i-0.5)*COVERED_SCALE_X, 
                           TERRAIN_Y*SCALE+MARGIN-(j+0.5)*COVERED_SCALE_Y);
                glVertex2d(MARGIN+ (i+0.5)*COVERED_SCALE_X, 
                           TERRAIN_Y*SCALE+MARGIN-(j+0.5)*COVERED_SCALE_Y);
                glVertex2d(MARGIN+ (i+0.5)*COVERED_SCALE_X, 
                           TERRAIN_Y*SCALE+MARGIN-(j-0.5)*COVERED_SCALE_Y);	
            }
        }
        glEnd();			// Done Drawing The Player
        //glDisable(GL_ALPHA_TEST);
    }
  
    // --------------------------------------------------------------------
    // drawTargetTrajectory
    // --------------------------------------------------------------------
    void drawTargetTrajectory() {
        if (!displayCurrentTrajectory_) return;
        glColor3f(0.0f,0.0f,1.0f);
        glBegin(GL_LINE_STRIP);	
        Trajectory::iterator it=targetTrajectory_.begin();
        for(;it!=targetTrajectory_.end();it++) {
            glVertex2d(MARGIN+(*it).x*SCALE, 
                       TERRAIN_Y*SCALE+MARGIN-(*it).y*SCALE);
        }
        glEnd();			
    }

    // --------------------------------------------------------------------
    // drawRealTrajectory
    // --------------------------------------------------------------------
    void drawRealTrajectory() 
    {
        if (!displayTrajectory_) return;
        glColor3f(1.0f,0.0f,0.0f);
        glBegin(GL_LINE_STRIP);
        Trajectory::iterator it=trajectory_.begin();
        for(;it!=trajectory_.end();it++) {
            glVertex2d(MARGIN+(*it).x*SCALE,
                       TERRAIN_Y*SCALE+MARGIN-(*it).y*SCALE);
        }
        glEnd();		
    }

    // --------------------------------------------------------------------
    // addPointToTrajectory
    // ------------------4--------------------------------------------------
    void addPointToTrajectory(Point pt) {
        trajectory_.push_back(pt);
    }

    // --------------------------------------------------------------------
    // drawRobot
    // --------------------------------------------------------------------
    void drawRobot(Millimeter x, Millimeter y, Millimeter t)
    {
	if (x<0) return;
        glColor3f(0.0f,0.0f,0.0f);
        glPushMatrix();
        glTranslatef(MARGIN+x*SCALE, 
                     MARGIN+TERRAIN_Y*SCALE-y*SCALE,0);
        glRotatef(-r2d(t),0,0,1);
        glBegin(GL_LINE_STRIP);
        glVertex2d(150*SCALE, 150*SCALE);
        glVertex2d(-150*SCALE, 150*SCALE);
        glVertex2d(-150*SCALE, -150*SCALE);
        glVertex2d(150*SCALE, -150*SCALE);
        glVertex2d(150*SCALE, 150*SCALE);
        glEnd();
        glBegin(GL_LINE_STRIP);
        glVertex2d(150*SCALE, 0*SCALE);
        glVertex2d(0*SCALE, 50*SCALE);
        glVertex2d(0*SCALE, -50*SCALE);
        glVertex2d(150*SCALE, 0*SCALE);
        glEnd();

    
        // arm left
        if (VIEWER3D->armLeft_) {
            glBegin(GL_LINES);
            glVertex2d(0*SCALE, -150*SCALE);
            glVertex2d(0*SCALE, -300*SCALE);
            glEnd();
        }
        // armRight
        if (VIEWER3D->armRight_) {
            glBegin(GL_LINES);
            glVertex2d(0*SCALE, 150*SCALE);
            glVertex2d(0*SCALE, 300*SCALE);
            glEnd();
        }
        // lowerArmOut_
        if (VIEWER3D->lowerArmOut_) {
            glBegin(GL_LINES);
            glVertex2d(150*SCALE, 150*SCALE);
            glVertex2d(250*SCALE, 250*SCALE);
            glEnd();
        }
        glBegin(GL_LINES);
        glVertex2d((-100+(40*cos(t-viewer_->gonioDirection_)))*SCALE, 
                   (-100+(40*sin(t-viewer_->gonioDirection_)))*SCALE);
        glVertex2d((-100-(40*cos(t-viewer_->gonioDirection_)))*SCALE, 
                   (-100-(40*sin(t-viewer_->gonioDirection_)))*SCALE);
        glEnd();
        glPopMatrix();
    }
  
    // --------------------------------------------------------------------
    // drawObstacle
    // --------------------------------------------------------------------
    void drawObstacle(Obstacle     const& obstacle,
                      ObstacleType obstacleType) 
    {
	if (obstacle.center.x<10) return;
	if (obstacleType == ALL_OBSTACLES) {
	    obstacleType = obstacle.type;
	}
	
	if (obstacleType==COCONUT_OBSTACLE) {
	    drawPalmTree(obstacle.center.x, obstacle.center.y, false);
	    return;
	}
        //glEnable(GL_ALPHA_TEST);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        switch(obstacleType) {
	case ENV_OBSTACLE:
            glColor4f(COLOR_ENV_OBSTACLE);
            break;
        case BUMPER_OBSTACLE:
            glColor4f(COLOR_BUMPER_OBSTACLE);
            break;
        case USER_OBSTACLE:
            glColor4f(COLOR_USER_OBSTACLE);
            break;
        case COCONUT_OBSTACLE:
            glColor4f(COLOR_DETECTED_COCONUT);
            break;
        case TRAJECTORY_OBSTACLE:
            glColor4f(COLOR_TRAJEC_OBSTACLE);
            break;
        case ALL_OBSTACLES:
        default:
            glColor4f(COLOR_TRAJEC_OBSTACLE);
            break;
        }
        GLUquadric* quad=gluNewQuadric();
        glPushMatrix ();
	glTranslatef(MARGIN+obstacle.center.x*SCALE, 
                     MARGIN+TERRAIN_Y*SCALE-obstacle.center.y*SCALE,0);
       
        gluDisk(quad, 0, obstacle.radius*SCALE, 10,5);
        glPopMatrix();

        gluDeleteQuadric(quad);
        //glDisable(GL_ALPHA_TEST);
    }
    
    // --------------------------------------------------------------------
    // drawDetectedBall
    // --------------------------------------------------------------------
    void drawDetectedBall(Ball const &ball) 
    {
	if (ball.center.x<10) return;
        //glEnable(GL_ALPHA_TEST);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        GLUquadric* quad=gluNewQuadric();
        glPushMatrix ();
        glColor4f(COLOR_DETECTED_BALL);
        glTranslatef(MARGIN+ball.center.x*SCALE, 
                     MARGIN+TERRAIN_Y*SCALE-ball.center.y*SCALE,0);
	glScalef(1.,1.5,1.);
        gluDisk(quad, 0, BALLON_RAYON*SCALE, 10,5);
        glPopMatrix();

        gluDeleteQuadric(quad);
        //glDisable(GL_ALPHA_TEST);
    }

    // --------------------------------------------------------------------
    // drawAsservissementBall
    // --------------------------------------------------------------------
    void drawAsservissementBall(Ball const &ball) 
    {
	if (ball.center.x<10) return;
        //glEnable(GL_ALPHA_TEST);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        GLUquadric* quad=gluNewQuadric();
        glPushMatrix ();
        glColor4f(COLOR_ASSERVISSEMENT_BALL);
        glTranslatef(MARGIN+ball.center.x*SCALE, 
                     MARGIN+TERRAIN_Y*SCALE-ball.center.y*SCALE,0);
	glScalef(1.,1.5,1.);
        gluDisk(quad, 0, BALLON_RAYON*SCALE, 10,5);
        glPopMatrix();

        gluDeleteQuadric(quad);
        //glDisable(GL_ALPHA_TEST);
    }

    // --------------------------------------------------------------------
    // drawObstacles
    // --------------------------------------------------------------------
    void drawObstacles() 
    {
        for(int i=0; i < OBSTACLE_TYPE_NBR; i++) {
            ListOfObstacles::iterator it=viewer_->obstacles_[i].begin();
            for(;it != viewer_->obstacles_[i].end(); it++) {
                if ((void*)(&(*it))!=NULL) drawObstacle(*it, (ObstacleType)i);
            }
        }
    }

    // --------------------------------------------------------------------
    // drawDetectedBalls
    // --------------------------------------------------------------------
    void drawDetectedBalls() 
    {
	int k=0;
        for(int i=0; i < OBSTACLE_TYPE_NBR; i++) {
            ListOfBalls::iterator it=viewer_->ballsDetected_.begin();
	    // latest balls are put front
            for(;it != viewer_->ballsDetected_.end() 
		    && k++<MAP_MAX_BALL_DISPLAYED;  it++) {
                drawDetectedBall(*it);
            }
        }
    }

    // --------------------------------------------------------------------
    // drawAsservissementBalls
    // --------------------------------------------------------------------
    void drawAsservissementBalls() 
    {
        for(int i=0; i < OBSTACLE_TYPE_NBR; i++) {
            ListOfBalls::iterator it=viewer_->ballsAsservissement_.begin();
            for(;it != viewer_->ballsAsservissement_.end(); it++) {
                drawAsservissementBall(*it);
            }
        }
    }

    // --------------------------------------------------------------------
    // drawGonioBalise
    // --------------------------------------------------------------------
    void drawGonioBalise() 
    {
	if (viewer_->baliseId_ == GONIO_BALISE_NBR) return;
        glColor3f(0.0f,1.0f,1.0f);
	Point pt1=viewer_->gonioCenter_;
	Point pt2=Point(viewer_->gonioCenter_.x+1000*cos(viewer_->gonioDirection_),
			viewer_->gonioCenter_.y+1000*sin(viewer_->gonioDirection_));
	
        glBegin(GL_LINES);
        glVertex2d(MARGIN+pt1.x*SCALE,
		   TERRAIN_Y*SCALE+MARGIN-pt1.y*SCALE);
        glVertex2d(MARGIN+pt2.x*SCALE,
		   TERRAIN_Y*SCALE+MARGIN-pt2.y*SCALE);
        glEnd();
	// point de depart
	GLUquadric* quad=gluNewQuadric();
	glPushMatrix();
	glTranslatef(MARGIN+pt1.x*SCALE, 
		     MARGIN+(TERRAIN_Y-pt1.y)*SCALE,0);
	gluDisk(quad, 0, 25*SCALE, 5,5);
	glPopMatrix(); 
	gluDeleteQuadric(quad);	

	// fleche d'arrivee
      Point pt3 = Point(pt2.x-100*cos(viewer_->gonioDirection_+M_PI/5),
			pt2.y-100*sin(viewer_->gonioDirection_+M_PI/5));
      Point pt4 = Point(pt2.x-100*cos(viewer_->gonioDirection_-M_PI/5),
			pt2.y-100*sin(viewer_->gonioDirection_-M_PI/5));

      glBegin(GL_LINES);
      glVertex2d(MARGIN+pt2.x*SCALE, 
		 TERRAIN_Y*SCALE+MARGIN-pt2.y*SCALE);
      glVertex2d(MARGIN+pt3.x*SCALE, 
		 TERRAIN_Y*SCALE+MARGIN-pt3.y*SCALE);
      glVertex2d(MARGIN+pt2.x*SCALE, 
		 TERRAIN_Y*SCALE+MARGIN-pt2.y*SCALE);
      glVertex2d(MARGIN+pt4.x*SCALE, 
		 TERRAIN_Y*SCALE+MARGIN-pt4.y*SCALE);
      glEnd();	
    }
    
    void drawLegend(Millimeter x, Millimeter y, Radian t) {
	text_->draw(5,(int)(TERRAIN_Y*SCALE+4*MARGIN-3),
		    "x=%3d, y=%3d, t=%3d, time=%d",
		    (int)x/10, (int)y/10, r2d(t)%360, 
		    (int)(viewer_->time_/1000.));

	GLUquadric* quad=gluNewQuadric();
	Millimeter CIRCLE_RAYON=50;
	int X=5,Y=(int)(TERRAIN_Y*SCALE+6*MARGIN);
        glPushMatrix ();
        glColor4f(COLOR_DETECTED_BALL);
        glTranslatef(X,Y-MARGIN,0);
	glScalef(1.,1.5,1.);
        gluDisk(quad, 0, BALLON_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-3, "ball");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_ENV_OBSTACLE);
        glTranslatef(X,Y-MARGIN,0);
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-3, "env");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_DETECTED_COCONUT);
        glTranslatef(X,Y-MARGIN,0);
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-3, "coco");

	X=5; Y+=2*MARGIN;
	glPushMatrix ();
        glColor4f(COLOR_BUMPER_OBSTACLE);
        glTranslatef(X,Y-MARGIN,0);
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-2, "Bump");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_USER_OBSTACLE);
        glTranslatef(X,Y-MARGIN,0);
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-2, "User");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_TRAJEC_OBSTACLE);
        glTranslatef(X,Y-MARGIN,0);
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-2, "Trajec");

	X=5; Y+=2*MARGIN;
	glPushMatrix ();
        glColor4f(COLOR_GRID_VISITED);
        glTranslatef(X,Y-MARGIN,0);
        glBegin(GL_QUADS);
	   glVertex2d(50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, -50*SCALE);
	   glVertex2d(50*SCALE, -50*SCALE);	
	glEnd();
	glPopMatrix();
	text_->draw(X+20,Y-2, "Visit");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_GRID_BALL);
        glTranslatef(X,Y-MARGIN,0);
	glBegin(GL_QUADS);
	   glVertex2d(50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, -50*SCALE);
	   glVertex2d(50*SCALE, -50*SCALE);	
	glEnd();
        glPopMatrix();
	text_->draw(X+20,Y-2, "Ball");

	X+=100;
	glPushMatrix ();
        glColor4f(COLOR_GRID_OBSTACLE);
        glTranslatef(X,Y-MARGIN,0);
	glBegin(GL_QUADS);
	   glVertex2d(50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, 50*SCALE);	
	   glVertex2d(-50*SCALE, -50*SCALE);
	   glVertex2d(50*SCALE, -50*SCALE);	
	glEnd();
        gluDisk(quad, 0, CIRCLE_RAYON*SCALE, 10,5);
        glPopMatrix();
	text_->draw(X+20,Y-2, "Obst");

	gluDeleteQuadric(quad);
    }

} // namespace

// --------------------------------------------------------------------
// initRobotMap
// --------------------------------------------------------------------
void initRobotMap(int winId, Viewer3D* viewer) {
    if (!init_) {
        text_=new ViewerText(10, ViewerColor(1,0,0,1));
        winId_=winId;
	viewer_=viewer;
        glEnable(GL_BLEND);
        glShadeModel (GL_SMOOTH);
        glClearColor (0.0, 0.0, 0.0, 0.0);
        glClear (GL_COLOR_BUFFER_BIT);
        //glutInitDisplayMode( GLUT_RGB );
#ifdef linux
        glutCreateMenu (menuRobotMap);
        glutAddMenuEntry ("Afficher la zone couverte", 1);
        glutAddMenuEntry ("Afficher la trajectoire du robot", 2);
        glutAddMenuEntry ("Afficher la destination du robot", 3);
        glutAttachMenu (GLUT_RIGHT_BUTTON);
#endif
        init_=true;
    
    }
    resetRobotMap();
}

// --------------------------------------------------------------------
// resetRobotMap
// --------------------------------------------------------------------
void resetRobotMap()
{
    trajectory_.clear();
    targetTrajectory_.clear();
    int i,j;
    for(i=0;i<COVERED_X;i++) {
        for(j=0;j<COVERED_Y;j++) {
            coveredZone_[i][j]=0;
        }
    }
    date_=0;
}

// --------------------------------------------------------------------
// displayRobotMap
// --------------------------------------------------------------------
void displayRobotMap(void) {
    //Lock localLock(&viewer3DrepositoryLock);
    if (winId_==0) return;
    Millimeter x,y;
    Radian t;
    if (viewer_->enableRobotPoint_) {
        viewer_->getRobotPosition(x,y,t);
        setCoveredZone(x,y,t);
        addPointToTrajectory(Point(x,y));
    }
    
#ifdef linux
    glutSetWindow(winId_);
#endif
    // dessine l'arriere plan
    drawBG();
    glEnable(GL_ALPHA_TEST);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    
    // dessine les trajectoires
    if (viewer_->enableRobotPoint_) {
        drawCoverZone();
        drawTargetTrajectory();
        drawRealTrajectory();
    }

    drawGrid(viewer_->grid_);

    // dessine les balles
    for(unsigned int i=0; i<BALLON_NBR; ++i) {
        if (viewer_->balls_ && !viewer_->balls_[i].inRobot) {
            drawBall(viewer_->balls_[i].center.x, viewer_->balls_[i].center.y);
        }
    }

    // dessine les obstacles detectes par le robot
    drawObstacles();
    drawDetectedBalls();
    drawAsservissementBalls();
    // dessine le robot
  
    if (viewer_->enableRobotPoint_) {
        drawRobot(viewer_->robotRP_.center.x, 
                  viewer_->robotRP_.center.y, 
                  viewer_->robotRP_.direction);
        drawEnnemy(viewer_->ennemyRP_.center.x, 
                   viewer_->ennemyRP_.center.y, 
                   viewer_->ennemyRP_.direction);
        drawEnnemyRS(viewer_->ennemyRS_.center.x, 
                     viewer_->ennemyRS_.center.y, 
                     viewer_->ennemyRS_.direction);
        drawRobotRS(viewer_->robotRS_.center.x, 
                    viewer_->robotRS_.center.y, 
                    viewer_->robotRS_.direction);
    }
  
    // dessine les 2 cocotier
    drawPalmTree(viewer_->coconut_.x, 
                 viewer_->coconut_.y);
    drawPalmTree(TERRAIN_LONGUEUR-viewer_->coconut_.x, 
                 TERRAIN_LARGEUR-viewer_->coconut_.y);

    // poteaux d'enbut
    drawPalmTree(300, 750);
    drawPalmTree(300, 1350);
    drawPalmTree(2700, 750);
    drawPalmTree(2700, 1350);

    // dessine la direction de la balise
    drawGonioBalise();
    
    drawLegend(x,y,t);
    Viewer3D::movieScreenShot(VIEWER_SCREEN_MAP);
#ifdef linux
    glutSwapBuffers() ;
#endif
    //Viewer3D::movieScreenShot(VIEWER_SCREEN_MAP);
    //fprintf(stdout, "Map\n");
}

// --------------------------------------------------------------------
// keyboardRobotMap
// --------------------------------------------------------------------
void keyboardRobotMap (unsigned char key, int x, int y){
    Point pt((mouseScaleX_*x-MARGIN)/SCALE,
	     (MARGIN+TERRAIN_Y*SCALE-mouseScaleY_*y)/SCALE);
  
    printf("Point(%d %d) -> point(%d %d)\n",
	   x, y,(int) pt.x, (int)pt.y);
    switch (key) {
    case 'c':
	Viewer3D::screenShot("robotMap");
	break;
    case 'a':
	SIMULATOR->setEnnemyRPPosition(pt, 0);
	break;
    case 'z':
	SIMULATOR->setEnnemyRSPosition(pt, 0);
	break;
    case 'e':
	SIMULATOR->setRSPosition(pt, 0);
	break;
    default:
	break;
    }
}

// --------------------------------------------------------------------
// menuRobotMap
// --------------------------------------------------------------------
void menuRobotMap (int value){
    switch(value) {
    case 1:
        displayCovered_=!displayCovered_;
        break;
    case 4:
        displayGrid_=!displayGrid_;
        break;
    case 2:
        displayTrajectory_=!displayTrajectory_;
        break;
    case 3:
        displayCurrentTrajectory_=!displayCurrentTrajectory_;
        break;
    }
}

// --------------------------------------------------------------------
// reshapeRobotMap
// --------------------------------------------------------------------
void reshapeRobotMap(int w, int h){
    //Lock localLock(&viewer3DrepositoryLock);
    /* glViewport(0,0,
       (GLsizei) (2*MARGIN+TERRAIN_X*SCALE), 
       (GLsizei) (2*MARGIN+TERRAIN_Y*SCALE)); */
  if (winId_==0) return;
  
#ifdef linux
    glutSetWindow(winId_);
#endif
    glViewport(0,0,
               (GLsizei) (w), 
               (GLsizei) (h));
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //gluOrtho2D(0.0, (GLfloat) 310.f, (GLfloat) 220.f, 1.f);
    glOrtho(0.0f,(2*MARGIN+TERRAIN_X*SCALE), (10*MARGIN+TERRAIN_Y*SCALE),0.0f,-1.0f,1.0f);
    /* gluLookAt (1500*SCALE, 1050*SCALE, 300, 
	       1500*SCALE, 1050*SCALE, 0, 
	       1, 0, 0); */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    mouseScaleX_=(2.*MARGIN+TERRAIN_X*SCALE)/w;
    mouseScaleY_=(8.*MARGIN+TERRAIN_Y*SCALE)/h;
   
}

// --------------------------------------------------------------------
// idleRobotMap
// --------------------------------------------------------------------
void idleRobotMap(void){
    //		glutPostRedisplay();
}

// --------------------------------------------------------------------
// timerRobotMap
// --------------------------------------------------------------------
void timerRobotMap(int v)
{
 
}

// --------------------------------------------------------------------
// robotMapSetTargetTrajectory
// --------------------------------------------------------------------
void robotMapSetTargetTrajectory(Trajectory const& t)
{
    targetTrajectory_ = t;
}

// --------------------------------------------------------------------
// robotMapSetCoveredDelay
// --------------------------------------------------------------------
void robotMapSetCoveredDelay(double delay)
{
    coveredDelay_=delay;
}

