#include <stdio.h>
#include <GL/glut.h>
#include <math.h>

#ifndef M_PI
#define M_PI (3.14)
#define M_PI_2 (M_PI/2.)
#define M_3_PI_2 (M_PI_2*3.)
#endif

#include "robotView.h"
#include "gltools.h"
#include "viewer3D.h"
#include "mthread.h"

extern pthread_mutex_t viewer3DrepositoryLock;

// -------------------------------------------------------------------------
// Private namespace
// -------------------------------------------------------------------------
namespace {

  GLdouble xEye=1500*SCALE3D, yEye=-3000*SCALE3D, zEye=350*SCALE3D; 
  GLdouble xCenter=1500*SCALE3D, yCenter=1050*SCALE3D, zCenter=0*SCALE3D;
  GLdouble xUp=0.1, yUp=0.1, zUp=1, dirR=M_PI_2;
  int W, H;
#define ZNEAR 1
#define ZFAR 10000
#define FOV_START 60
  int FOVY = FOV_START;
  GLdouble dAspect=1;

  int winId_=0;
  Viewer3D* viewer_=NULL;
  bool init_=false;

  // --------------------------------------------------------------------------
  // SetRobotPosition -- Eyes direction
  // --------------------------------------------------------------------------
  void SetRobotPosition(Millimeter cx, Millimeter cy, Millimeter theta)
  {
    xEye = cx*SCALE3D;
    yEye = cy*SCALE3D;
    
    double DIST=2000*SCALE3D;
    xCenter = xEye+DIST*cos(theta);
    yCenter = yEye+DIST*sin(theta);
    zCenter = 0;
    
    if(xEye != xCenter){
      dirR = atan((yEye-yCenter)/(xEye-xCenter));
      if(xCenter<xEye) dirR += M_PI;
    } else{
      if(yCenter>yEye) dirR = M_PI_2;
      else dirR = M_PI_2*3;
    }
  } // SetRobotPosition

} // namespace


// -------------------------------------------------------------------------
// initRobotView
// -------------------------------------------------------------------------
void initRobotView(int winId, Viewer3D* viewer) 
{
  if (!init_) {
    winId_=winId;
    viewer_=viewer;
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    
    glShadeModel (GL_SMOOTH);
    
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glClearColor (0.0, 0.0, 0.0, 0.0);
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glutCreateMenu (menuRobotView);
    //glutAddMenuEntry ("Camra adapte", 1);
    glutAttachMenu (GLUT_RIGHT_BUTTON);
    init_=true;
  }
  resetRobotView();
}

// -------------------------------------------------------------------------
// resetRobotView
// -------------------------------------------------------------------------
void resetRobotView() 
{

}

// -------------------------------------------------------------------------
// displayRobotView
// -------------------------------------------------------------------------
void displayRobotView(void) 
{
  //Lock localLock(&viewer3DrepositoryLock);
  if (winId_==0) return;
  glutSetWindow(winId_);
  SetRobotPosition(viewer_->robotRP_.center.x, 
		   viewer_->robotRP_.center.y, 
		   viewer_->robotRP_.direction);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(FOVY, dAspect, ZNEAR, ZFAR);
  gluLookAt (xEye, yEye, zEye,
	     xCenter, yCenter, zCenter, 
	     cos(dirR), sin(dirR), 0);
  //gluLookAt (xEye, yEye, zEye, xCenter, yCenter, zCenter, xUp, yUp, zUp);
  glMatrixMode(GL_MODELVIEW);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glLoadIdentity ();
  viewer_->drawBG();
  viewer_->drawPalmTrees();
  viewer_->drawBalls();
  viewer_->drawEnnemy();
  viewer_->drawEnnemyRS();
  viewer_->drawRobotRS();
  glutSwapBuffers() ;
  Viewer3D::movieScreenShot(VIEWER_SCREEN_ROBOT_VISION);
  //fprintf(stdout, "View\n");
}

// -------------------------------------------------------------------------
// keyboardRobotView
// -------------------------------------------------------------------------
void keyboardRobotView (unsigned char key, int x, int y) 
{ 
  switch (key) {
    case 'c':
      Viewer3D::screenShot("robotView");
      break;
  default:
    break;
  }
}

// -------------------------------------------------------------------------
// menuRobotView
// -------------------------------------------------------------------------
void menuRobotView (int value) 
{

}

// -------------------------------------------------------------------------
// reshapeRobotView
// -------------------------------------------------------------------------
void reshapeRobotView(int w, int h) 
{
#ifdef linux
    glutSetWindow(winId_);
#endif
  //Lock localLock(&viewer3DrepositoryLock);
  glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
  
  // quivalent du choix d'une lentille pour la camera	
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective(30.0, (GLfloat) w/(GLfloat) h, 1.0, 800.0);
  
  glMatrixMode(GL_MODELVIEW);
}

// -------------------------------------------------------------------------
// idleRobotView
// -------------------------------------------------------------------------
void idleRobotView(void) 
{

}

// -------------------------------------------------------------------------
// timerRobotView
// -------------------------------------------------------------------------
void timerRobotView(int v) 
{
	
}
