#include <stdio.h>
#include <math.h>
#include <assert.h>

#ifndef VIEWER_DISACTIVATED 
#include <GL/glut.h>
#include "mthread.h"
#include "log.h"

#ifndef M_PI
#define M_PI (3.14)
#define M_PI_2 (M_PI/2.)
#define M_3_PI_2 (M_PI_2*3.)
#endif

#include "robotMap.h"
#include "robotView.h"
#include "robot3d.h"
#include "robotData.h"
#include "logPlayer.h"
#include "texture.h"
#include "gltools.h"
#endif // VIEWER_DISACTIVATED

#include "viewer3D.h"

Viewer3D* Viewer3D::viewer_=NULL;

void (*viewer3DDisplayCB)()=NULL;

#ifndef VIEWER_DISACTIVATED 
pthread_mutex_t viewer3DrepositoryLock = PTHREAD_MUTEX_INITIALIZER;

namespace {
    bool enablePoint_=false;
}
#endif // VIEWER_DISACTIVATED

// ---------------------------------------------------------------------------
// Viewer3D::Viewer3D
// ---------------------------------------------------------------------------
Viewer3D::Viewer3D(const RobotConfig* config, bool createThread) : 
    RobotComponent("Viewer3D", CLASS_VIEWER_3D, config),
    robotRP_(450, 300, 0), armRight_(false), armLeft_(false), 
    entryOpened_(true), time_(0), enableRobotPoint_(false), balls_(NULL),
    robotRS_(-10,0,0), ennemyRP_(-10,0,0), ennemyRS_(-10,0,0), 
    useTexture_(false), createThread_(createThread), texture_(NULL)
{
    assert(!viewer_);
    viewer_=this;
#ifndef VIEWER_DISACTIVATED 
    enableRobotPoint_=false;
    enablePoint_=enableRobotPoint_;
    char* argv[]={"Robot"};
    texture_ = new Texture();
    if (config) {
        createWindows(1, argv, 
                      config->viewerMap, 
                      config_->viewerCamera, 
                      config->viewer3d, 
                      config->viewerData,
                      config->viewerLogPlayer);
        setCoveredZoneEraseTime(config->viewerOverlapTimerConstante);
    }
    LOG_OK("Initialisation termine\n");
#endif
}

// ---------------------------------------------------------------------------
// Viewer3D::~Viewer3D
// ---------------------------------------------------------------------------
Viewer3D::~Viewer3D()
{
#ifndef VIEWER_DISACTIVATED 
    viewerData_.clear();
    if (createThread_) MTHREAD_CANCEL(thread_);
    if (texture_) delete texture_; 
#endif
    viewer_=NULL;
}

// ---------------------------------------------------------------------------
// Viewer3D::instance
// ---------------------------------------------------------------------------
Viewer3D* Viewer3D::instance()
{
    //assert(viewer_);
    return viewer_;
}
// ---------------------------------------------------------------------------
// Met a jour la position du robot secondaire ennemy
// ---------------------------------------------------------------------------
void Viewer3D::setGonioBalise(GonioBalise baliseId,
			      Point const& gonioCenter,
			      Radian const& direction)
{
#ifndef VIEWER_DISACTIVATED 
    baliseId_=baliseId;
    gonioCenter_=gonioCenter;
    gonioDirection_=direction;
#endif // VIEWER_DISACTIVATED 
}
// ---------------------------------------------------------------------------
// Met a jour la position du robot secondaire ennemy
// ---------------------------------------------------------------------------
void Viewer3D::setBallListDetected(ListOfBalls const& balls)
{
#ifndef VIEWER_DISACTIVATED 
    ballsDetected_=balls;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Met a jour la position du robot secondaire ennemy
// ---------------------------------------------------------------------------
void Viewer3D::setBallAsservissement(ListOfBalls const& balls)
{
#ifndef VIEWER_DISACTIVATED 
    ballsAsservissement_=balls;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Met a jour la grille utilisee pour generer des trajectoires
// ---------------------------------------------------------------------------
void Viewer3D::setGrid(LogGrid* grid)
{
#ifndef VIEWER_DISACTIVATED 
    grid_=grid;
#endif // VIEWER_DISACTIVATED 
}
// ---------------------------------------------------------------------------
// Viewer3D::setRobotActuators
// ---------------------------------------------------------------------------
// Met a jour la position des actionneurs du robot 
// ---------------------------------------------------------------------------
void Viewer3D::setRobotActuators(bool armLeftOut, 
				 bool armRightOut, 
				 bool entryOpened,
				 bool lowerArmOut,
				 bool backDoorClosed)
{
#ifndef VIEWER_DISACTIVATED 
    armLeft_=armLeftOut;
    armRight_=armRightOut;
    entryOpened_=entryOpened;
    lowerArmOut_=lowerArmOut;
    backDoor_=backDoorClosed;
#endif // VIEWER_DISACTIVATED 
}


// ---------------------------------------------------------------------------
// Viewer3D::getRobotPosition
// ---------------------------------------------------------------------------
void Viewer3D::getRobotPosition(double &x, double &y, double &theta)
{
#ifndef VIEWER_DISACTIVATED 
    x=robotRP_.center.x;
    y=robotRP_.center.y;
    theta=robotRP_.direction;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setRobotPosition
// ---------------------------------------------------------------------------
void Viewer3D::setRobotPosition(Millimeter x, Millimeter y, Radian theta)
{
#ifndef VIEWER_DISACTIVATED 
    Lock localLock(&viewer3DrepositoryLock, config());
    robotRP_.center.x=x;
    robotRP_.center.x=y;
    robotRP_.direction=theta;
    enableRobotPoint_=enablePoint_;
#endif // VIEWER_DISACTIVATED 
}

#ifndef VIEWER_DISACTIVATED 

// ---------------------------------------------------------------------------
// private namespace
// ---------------------------------------------------------------------------
namespace {
    extern "C" {
        void* glutThreadBody(void* data) 
        {
            glutMainLoop();
            return NULL;
        }
    }
 
    void emptyFunction() {};

    inline void display() {
        displayRobot3D();
        displayRobotMap();
        displayRobotView();
        displayRobotData();
        displayPlayerControl();
        if (viewer3DDisplayCB) viewer3DDisplayCB();
    }

    void timerFunction(int value) {
        display();
        glutTimerFunc(100, timerFunction, value);
    }
}
#endif

// ---------------------------------------------------------------------------
// Viewer3D::createWindows
// ---------------------------------------------------------------------------
void Viewer3D::createWindows(int argc, char*argv[], 
			     bool carte, bool robotCamera, 
			     bool robot3D, bool robotData,
			     bool robotLogPlayer) 
{
#ifndef VIEWER_DISACTIVATED 
    LOG_FUNCTION();
    glutInit(&argc, argv); 
    glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH ); 
    loadGLTextures();
    buildFont();
    viewerData_.clear();
    int winId=0;
    // cration des fentres
    // Carte du terrain
    if (carte) {
        glutInitWindowPosition(0,320); 
        glutInitWindowSize(320,226); 
        winId=glutCreateWindow ("Carte"); 
        glutReshapeFunc(reshapeRobotMap);
        glutDisplayFunc(emptyFunction);
        glutIdleFunc (idleRobotMap);
        glutKeyboardFunc(keyboardRobotMap);
        initRobotMap(winId, this);
        texture_->init();
    }

    if (robotCamera) {
        // Vue depuis le robot
        glutInitWindowPosition(335,320); 
        glutInitWindowSize(160,120); 
        winId=glutCreateWindow ("Vision du robot"); 
        glutReshapeFunc(reshapeRobotView);
        glutDisplayFunc(emptyFunction); 
        glutIdleFunc (idleRobotView);
        glutKeyboardFunc(keyboardRobotView);
        initRobotView(winId, this);
        texture_->init();
    }

    if (robot3D) {
        // vue cool en 3D
        glutInitWindowPosition(00,00); 
        glutInitWindowSize(500, 290); 
        winId=glutCreateWindow ("Vue 3D"); 
        glutReshapeFunc(reshapeRobot3D);
        glutDisplayFunc(emptyFunction); 
        glutIdleFunc (idleRobot3D);
        glutKeyboardFunc(keyboardRobot3D);
        initRobot3D(winId, this);
        texture_->init();
    }

    if (robotData) {
        // vue cool en 3D
        glutInitWindowPosition(0,570); 
        glutInitWindowSize(500, 130); 
        winId=glutCreateWindow ("Donnes du robot"); 
        glutReshapeFunc(reshapeRobotData);
        glutDisplayFunc(emptyFunction); 
        glutIdleFunc (idleRobotData);
        glutKeyboardFunc(keyboardRobotData);
        initRobotData(winId, this);
        texture_->init();
    }
    if (robotLogPlayer) {
        // vue cool en 3D
        glutInitWindowPosition(500,0); 
        glutInitWindowSize(276, 100); 
        winId=glutCreateWindow ("LogPlayer"); 
        glutReshapeFunc(reshapePlayerControl);
        glutDisplayFunc(emptyFunction); 
        glutIdleFunc(emptyFunction);
        glutKeyboardFunc(keyboardPlayerControl);
        glutMouseFunc(mousePlayerControl);
        initPlayerControl(winId, this);
        texture_->init();
    }
    //  texture_->init();
    // create a timer that will update the display
    // create a thread for glutMainLoop which is a blocking function
    if (createThread_) {
        startThread();
    }
    init_=true;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::startThread
// ---------------------------------------------------------------------------
void Viewer3D::startThread()
{
#ifndef VIEWER_DISACTIVATED 
    glutTimerFunc(100, timerFunction, (int)this);
    MTHREAD_CREATE("Viewer", &thread_, NULL, glutThreadBody, NULL);
    createThread_=true;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::updateDisplay
// ---------------------------------------------------------------------------
void Viewer3D::updateDisplay(Millisecond time)
{
#ifndef VIEWER_DISACTIVATED 
    assert(init_); // createWindows must have been run first
    time_=time;
    // display will be updated by the timer functions
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::reset
// ---------------------------------------------------------------------------
bool Viewer3D::reset()
{
#ifndef VIEWER_DISACTIVATED 
    Lock localLock(&viewer3DrepositoryLock, config());
    LOG_FUNCTION();
    assert(init_);  // createWindows must have been run first
    time_=0;
    obstacles_[0].clear();
    enableRobotPoint_=false;
    resetRobot3D();
    resetRobotMap();
    resetRobotView();
    resetRobotData();
    return init_;
#else
    return true;
#endif
}

// ---------------------------------------------------------------------------
// Viewer3D::validate
// ---------------------------------------------------------------------------
bool Viewer3D::validate()
{
    return false;
}

// ---------------------------------------------------------------------------
// Viewer3D::setTargetTrajectory
// ---------------------------------------------------------------------------
void Viewer3D::setTargetTrajectory(Trajectory const& t) 
{
#ifndef VIEWER_DISACTIVATED 
    Lock localLock(&viewer3DrepositoryLock, config());
    robotMapSetTargetTrajectory(t);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setCoveredZoneEraseTime
// ---------------------------------------------------------------------------
void Viewer3D::setCoveredZoneEraseTime(Millisecond displayCycles)
{
#ifndef VIEWER_DISACTIVATED 
    Lock localLock(&viewer3DrepositoryLock, config());
    robotMapSetCoveredDelay(displayCycles);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setObstacleList
// ---------------------------------------------------------------------------
void Viewer3D::setObstacleList(ListOfObstacles const& obstacles,
                               ObstacleType type)
{
#ifndef VIEWER_DISACTIVATED 
    Lock localLock(&viewer3DrepositoryLock, config());
    obstacles_[type] = obstacles;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::registerViewerData
// ---------------------------------------------------------------------------
void Viewer3D::registerViewerData(ViewerData* data)
{
#ifndef VIEWER_DISACTIVATED 
    viewerData_.push_back(data);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setViewerDataTimeScale
// ---------------------------------------------------------------------------
void Viewer3D::setViewerDataTimeScale(Millisecond Min, Millisecond Max)
{
#ifndef VIEWER_DISACTIVATED 
    robotDataSetTimeScale(Min, Max);
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::enableRobotPoint
// ---------------------------------------------------------------------------
void Viewer3D::enableRobotPoint(bool enable)
{
#ifndef VIEWER_DISACTIVATED 
    if (enable==false) {
	enableRobotPoint_=enable;
    }
    enablePoint_=enable;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setCoconutPosition
// ---------------------------------------------------------------------------
void Viewer3D::setCoconutPosition(Point coconutCenter)
{
#ifndef VIEWER_DISACTIVATED 
    coconut_ = coconutCenter;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Viewer3D::setBallsPosition
// ---------------------------------------------------------------------------
void Viewer3D::setBallsPosition(Ball* balls)
{
#ifndef VIEWER_DISACTIVATED 
    balls_ = balls;
#endif // VIEWER_DISACTIVATED 
}
// ---------------------------------------------------------------------------
// Met a jour la position du robot
// ---------------------------------------------------------------------------
void Viewer3D::setRobotPosition(Position const& pos)
{
#ifndef VIEWER_DISACTIVATED 
    robotRP_= pos;
    enableRobotPoint_=enablePoint_;
#endif // VIEWER_DISACTIVATED 
}
// ---------------------------------------------------------------------------
// Met a jour la position du robot secondaire
// ---------------------------------------------------------------------------
void Viewer3D::setRobotRSPosition(Position const& pos)
{
#ifndef VIEWER_DISACTIVATED 
    robotRS_=pos;
#endif // VIEWER_DISACTIVATED 
}
// ---------------------------------------------------------------------------
// Met a jour la position du robot ennemy
// ---------------------------------------------------------------------------
void Viewer3D::setEnnemyPosition(Position const& pos)
{
#ifndef VIEWER_DISACTIVATED 
    ennemyRP_=pos;
#endif // VIEWER_DISACTIVATED 
}

// ---------------------------------------------------------------------------
// Met a jour la position du robot secondaire ennemy
// ---------------------------------------------------------------------------
void Viewer3D::setEnnemyRSPosition(Position const& pos)
{
#ifndef VIEWER_DISACTIVATED 
    ennemyRS_=pos;
#endif // VIEWER_DISACTIVATED 
}


// ===========================================================================
// Video et screenshot
// ===========================================================================
bool checkIfFileExist(const char* filename)
{
    FILE* fptr=fopen(filename, "rb");
    if (fptr) {
        fclose(fptr);
        return true;
    } else {
        return false;
    }
}

void Viewer3D::screenShot(const char* filename)
{
#ifndef VIEWER_DISACTIVATED 
    Image img;
    unsigned int width  = glutGet(GLUT_WINDOW_WIDTH);
    unsigned int height = glutGet(GLUT_WINDOW_HEIGHT);
    img.alloc(width, height);
    glReadPixels(0, 0, width, height,
                 GL_RGB, // format
                 GL_UNSIGNED_BYTE, // type
                 img.data);
    static char filename2[255];
    static int count=0;
    do {
        sprintf(filename2, 
                "../screenshot/%s_%.4d.bmp", filename, count++);
    } while(checkIfFileExist(filename2));
    if (BmpBitmap::save(img, filename2, false, true)) {
        printf("Take screenshot %s\n", filename2);
    }
#endif // VIEWER_DISACTIVATED 
}

MovieData Viewer3D::movieData_[VIEWER_SCREEN_NBR];
void Viewer3D::movieStart(const char* filename,
			  ViewerScreen screen)
{
#ifndef VIEWER_DISACTIVATED 
    int id=(int)screen;
    movieData_[id].run=true;
    static int count=0;
    do {
        sprintf(movieData_[id].filename, 
                "../movie/%s_%.4d_0000.bmp", filename, count++);
    } while(checkIfFileExist(movieData_[id].filename));
    printf("startMovie %s\n", movieData_[id].filename);
    movieData_[id].filename[strlen(movieData_[id].filename)-8]=0;
    movieData_[id].count=0;
#endif // VIEWER_DISACTIVATED 
}

void Viewer3D::movieStop(ViewerScreen screen)
{
#ifndef VIEWER_DISACTIVATED 
    movieData_[(int)screen].run=false;
    printf("stopMovie %s\n", movieData_[(int)screen].filename);
#endif // VIEWER_DISACTIVATED 
}

int Viewer3D::getMovieBaseName(ViewerScreen screen, 
                               char*filename, bool inc)
{
#ifndef VIEWER_DISACTIVATED 
    int id=(int)screen;
    sprintf(filename, "%s%.4d", 
            movieData_[id].filename, 
            movieData_[id].count);
    if (inc) {
        movieData_[id].count++;
        return movieData_[id].count-1;
    } else {
        return movieData_[id].count;
    }
#else 
    return 0;
#endif // VIEWER_DISACTIVATED 
}

void Viewer3D::movieScreenShot(ViewerScreen screen)
{
#ifndef VIEWER_DISACTIVATED 
    int id=(int)screen;
    if (!movieData_[(int)screen].run) return;
    unsigned int width  = glutGet(GLUT_WINDOW_WIDTH);
    unsigned int height = glutGet(GLUT_WINDOW_HEIGHT);
    movieData_[id].img.alloc(width, height);
    glReadPixels(0, 0, width-1, height-1,
		 GL_RGB, // format
		 GL_UNSIGNED_BYTE, // type
		 movieData_[id].img.data);
    static char filename[255];
    getMovieBaseName(screen, filename);
    strcat(filename, ".bmp");
    BmpBitmap::save(movieData_[id].img, filename, true, true);
#endif // VIEWER_DISACTIVATED 
}

void Viewer3D::setCameraPosition(Millimeter cx, 
				 Millimeter cy, 
				 Millimeter cz)
{
#ifndef VIEWER_DISACTIVATED 
    setCameraPosition3D(cx, cy, cz);
    LOG->simuSetCamera(cx, cy, cz);
#endif // VIEWER_DISACTIVATED 
}
